# Copyright (c) 2018-2026 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
# SPDX-License-Identifier: Apache-2.0
#
# vim: set noai syntax=python ts=4 sw=4:
"""Wait Wait Stats Panelist Data Retrieval Functions."""

from typing import Any

from mysql.connector import connect
from mysql.connector.connection import MySQLConnection
from mysql.connector.pooling import PooledMySQLConnection
from slugify import slugify

from wwdtm.guest.utility import GuestUtility
from wwdtm.host.utility import HostUtility
from wwdtm.panelist.appearances import PanelistAppearances
from wwdtm.panelist.statistics import PanelistStatistics
from wwdtm.panelist.utility import PanelistUtility
from wwdtm.scorekeeper.utility import ScorekeeperUtility
from wwdtm.validation import valid_int_id


class Panelist:
    """Panelist information retrieval class.

    Contains methods used to retrieve panelist information, including
    IDs, names, slug strings, appearances and scores.

    :param connect_dict: A dictionary containing database connection
        settings as required by MySQL Connector/Python
    :param database_connection: MySQL database connection object
    """

    def __init__(
        self,
        connect_dict: dict[str, Any] = None,
        database_connection: MySQLConnection | PooledMySQLConnection = None,
    ):
        """Class initialization method."""
        if connect_dict:
            self.connect_dict = connect_dict
            self.database_connection = connect(**connect_dict)
        elif database_connection:
            if not database_connection.is_connected():
                database_connection.reconnect()

            self.database_connection = database_connection

        self.appearances = PanelistAppearances(
            database_connection=self.database_connection
        )
        self.statistics = PanelistStatistics(
            database_connection=self.database_connection
        )
        self.utility = PanelistUtility(database_connection=self.database_connection)

    def retrieve_all(self) -> list[dict[str, Any]]:
        """Retrieve panelist information for all panelists.

        :return: A list of dictionaries containing panelist ID, name,
            slug string, gender and pronouns for each panelist
        """
        query = """
            SELECT p.panelistid AS id, p.panelist AS name, p.panelistslug AS slug,
            panelistgender AS gender
            FROM ww_panelists p
            WHERE panelistslug != 'multiple'
            ORDER BY panelist ASC;
            """
        cursor = self.database_connection.cursor(dictionary=True)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        panelists = []

        cursor = self.database_connection.cursor(dictionary=True)
        for row in results:
            query = """
                SELECT pn.pronouns
                FROM ww_panelistpronounsmap ppm
                JOIN ww_pronouns pn on pn.pronounsid = ppm.pronounsid
                WHERE ppm.panelistid = %s
                ORDER BY ppm.panelistpronounsmapid ASC;
                """
            cursor.execute(query, (row["id"],))
            pn_results = cursor.fetchall()

            panelists.append(
                {
                    "id": row["id"],
                    "name": row["name"],
                    "slug": row["slug"] if row["slug"] else slugify(row["name"]),
                    "gender": row["gender"],
                    "pronouns": (
                        [result["pronouns"] for result in pn_results]
                        if pn_results
                        else []
                    ),
                }
            )

        return panelists

    def retrieve_all_details(
        self, use_decimal_scores: bool = False
    ) -> list[dict[str, Any]]:
        """Retrieves panelist information, appearances and scores for all panelists.

        :param use_decimal_scores: A boolean to determine if decimal
            scores should be used and returned instead of integer scores
        :return: A list of dictionaries containing panelist ID, name,
            slug string, gender, pronouns, whether the panelist is
            also a guest, host or scorekeeper, scoring statistics and
            appearances for each panelist
        """
        query = """
            SELECT p.panelistid AS id, p.panelist AS name, p.panelistslug AS slug,
            p.panelistgender AS gender
            FROM ww_panelists p
            WHERE panelistslug != 'multiple'
            ORDER BY panelist ASC;
            """
        cursor = self.database_connection.cursor(dictionary=True)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        panelists = []
        _guest_utility = GuestUtility(database_connection=self.database_connection)
        _host_utility = HostUtility(database_connection=self.database_connection)
        _scorekeeper_utility = ScorekeeperUtility(
            database_connection=self.database_connection
        )

        cursor = self.database_connection.cursor(dictionary=True)
        for row in results:
            _slug = row["slug"] if row["slug"] else slugify(row["name"])
            query = """
                SELECT pn.pronouns
                FROM ww_panelistpronounsmap ppm
                JOIN ww_pronouns pn on pn.pronounsid = ppm.pronounsid
                WHERE ppm.panelistid = %s
                ORDER BY ppm.panelistpronounsmapid ASC;
                """
            cursor.execute(query, (row["id"],))
            pn_results = cursor.fetchall()
            panelists.append(
                {
                    "id": row["id"],
                    "name": row["name"],
                    "slug": _slug,
                    "gender": row["gender"],
                    "pronouns": (
                        [result["pronouns"] for result in pn_results]
                        if pn_results
                        else []
                    ),
                    "is_guest": bool(_guest_utility.slug_exists(guest_slug=_slug)),
                    "is_host": bool(_host_utility.slug_exists(host_slug=_slug)),
                    "is_scorekeeper": bool(
                        _scorekeeper_utility.slug_exists(scorekeeper_slug=_slug)
                    ),
                    "statistics": self.statistics.retrieve_statistics_by_id(
                        row["id"], include_decimal_scores=use_decimal_scores
                    ),
                    "bluffs": self.statistics.retrieve_bluffs_by_id(row["id"]),
                    "appearances": self.appearances.retrieve_appearances_by_id(
                        row["id"], use_decimal_scores=use_decimal_scores
                    ),
                }
            )

        return panelists

    def retrieve_all_ids(self) -> list[int]:
        """Retrieves all panelist IDs, sorted by panelist name.

        :return: A list of panelist IDs as integers
        """
        query = """
            SELECT panelistid FROM ww_panelists
            WHERE panelistslug != 'multiple'
            ORDER BY panelist ASC;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        return [v[0] for v in results]

    def retrieve_all_slugs(self) -> list[str]:
        """Retrieves all panelist slug strings, sorted by panelist name.

        :return: A list of panelist slug strings
        """
        query = """
            SELECT panelistslug FROM ww_panelists
            WHERE panelistslug != 'multiple'
            ORDER BY panelist ASC;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        return [v[0] for v in results]

    def retrieve_by_id(self, panelist_id: int) -> dict[str, Any]:
        """Retrieves panelist information.

        :param panelist_id: Panelist ID
        :return: A dictionary containing panelist ID, name, slug string,
            gender and pronouns
        """
        if not valid_int_id(panelist_id):
            return {}

        query = """
            SELECT p.panelistid AS id, p.panelist AS name, p.panelistslug AS slug,
            p.panelistgender AS gender
            FROM ww_panelists p
            WHERE p.panelistid = %s
            LIMIT 1;
            """
        cursor = self.database_connection.cursor(dictionary=True)
        cursor.execute(query, (panelist_id,))
        result = cursor.fetchone()
        cursor.close()

        if not result:
            return {}

        query = """
            SELECT pn.pronouns
            FROM ww_panelistpronounsmap ppm
            JOIN ww_pronouns pn on pn.pronounsid = ppm.pronounsid
            WHERE ppm.panelistid = %s
            ORDER BY ppm.panelistpronounsmapid ASC;
            """
        cursor = self.database_connection.cursor(dictionary=True)
        cursor.execute(query, (panelist_id,))
        results = cursor.fetchall()
        cursor.close()

        return {
            "id": result["id"],
            "name": result["name"],
            "slug": result["slug"] if result["slug"] else slugify(result["name"]),
            "gender": result["gender"],
            "pronouns": [result["pronouns"] for result in results] if results else [],
        }

    def retrieve_by_slug(self, panelist_slug: str) -> dict[str, Any]:
        """Retrieves panelist information.

        :param panelist_slug: Panelist slug string
        :return: A dictionary containing panelist ID, name, slug string,
            gender and pronouns
        """
        try:
            slug = panelist_slug.strip()
            if not slug:
                return {}
        except AttributeError:
            return {}

        id_ = self.utility.convert_slug_to_id(slug)
        if not id_:
            return {}

        return self.retrieve_by_id(id_)

    def retrieve_details_by_id(
        self, panelist_id: int, use_decimal_scores: bool = False
    ) -> dict[str, Any]:
        """Retrieves panelist information, appearances and scores.

        :param panelist_id: Panelist ID
        :param use_decimal_scores: A boolean to determine if decimal
            scores should be used and returned instead of integer scores
        :return: A dictionary containing panelist ID, name, slug string,
            gender, pronouns, whether the panelist is also a guest, host
            or scorekeeper, scoring statistics and appearances
        """
        if not valid_int_id(panelist_id):
            return {}

        info = self.retrieve_by_id(panelist_id)
        if not info:
            return {}

        _guest_utility = GuestUtility(database_connection=self.database_connection)
        _host_utility = HostUtility(database_connection=self.database_connection)
        _scorekeeper_utility = ScorekeeperUtility(
            database_connection=self.database_connection
        )

        info["is_guest"] = bool(_guest_utility.slug_exists(guest_slug=info["slug"]))
        info["is_host"] = bool(_host_utility.slug_exists(host_slug=info["slug"]))
        info["is_scorekeeper"] = bool(
            _scorekeeper_utility.slug_exists(scorekeeper_slug=info["slug"])
        )

        info["statistics"] = self.statistics.retrieve_statistics_by_id(
            panelist_id, include_decimal_scores=use_decimal_scores
        )
        info["bluffs"] = self.statistics.retrieve_bluffs_by_id(panelist_id)
        info["appearances"] = self.appearances.retrieve_appearances_by_id(
            panelist_id, use_decimal_scores=use_decimal_scores
        )

        return info

    def retrieve_details_by_slug(
        self, panelist_slug: str, use_decimal_scores: bool = False
    ) -> dict[str, Any]:
        """Retrieves panelist information, appearances and scores.

        :param panelist_slug: Panelist slug string
        :param use_decimal_scores: A boolean to determine if decimal
            scores should be used and returned instead of integer scores
        :return: A dictionary containing panelist ID, name, slug string,
            gender, pronouns, whether the panelist is also a guest, host
            or scorekeeper, scoring statistics and appearances
        """
        try:
            slug = panelist_slug.strip()
            if not slug:
                return {}
        except AttributeError:
            return {}

        id_ = self.utility.convert_slug_to_id(slug)
        if not id_:
            return {}

        return self.retrieve_details_by_id(id_, use_decimal_scores=use_decimal_scores)

    def retrieve_random_id(self) -> int:
        """Retrieves an ID for a random panelist.

        :return: ID for a random panelist.
        """
        query = """
            SELECT panelistid FROM ww_panelists
            WHERE panelistslug <> 'multiple'
            ORDER BY RAND()
            LIMIT 1;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query)
        result = cursor.fetchone()
        cursor.close()

        if not result:
            return None

        return result[0]

    def retrieve_random_slug(self) -> str:
        """Retrieves an slug string for a random panelist.

        :return: Slug string for a random panelist.
        """
        query = """
            SELECT panelistslug FROM ww_panelists
            WHERE panelistslug <> 'multiple'
            ORDER BY RAND()
            LIMIT 1;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query)
        result = cursor.fetchone()
        cursor.close()

        if not result:
            return None

        return result[0]

    def retrieve_random(self) -> dict[str, Any]:
        """Retrieves information for a random panelist.

        :return: A dictionary containing panelist ID, name, slug string
            and gender
        """
        _id = self.retrieve_random_id()

        if not _id:
            return None

        return self.retrieve_by_id(panelist_id=_id)

    def retrieve_random_details(
        self, use_decimal_scores: bool = False
    ) -> dict[str, Any]:
        """Retrieves information and appearances for a random panelist.

        :return: A dictionary containing panelist ID, name, slug string,
            gender, pronouns, whether the panelist is also a guest, host
            or scorekeeper, scoring statistics and appearances
        """
        _id = self.retrieve_random_id()

        if not _id:
            return None

        return self.retrieve_details_by_id(
            panelist_id=_id, use_decimal_scores=use_decimal_scores
        )



# Copyright (c) 2018-2026 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
# SPDX-License-Identifier: Apache-2.0
#
# vim: set noai syntax=python ts=4 sw=4:
"""Testing for object: :py:class:`wwdtm.panelist.Panelist`."""

import json
from pathlib import Path
from typing import Any

import pytest

from wwdtm.panelist import Panelist


@pytest.mark.skip
def get_connect_dict() -> dict[str, Any]:
    """Retrieves database connection settings.

    :return: A dictionary containing database connection
        settings as required by MySQL Connector/Python
    """
    file_path = Path.cwd() / "config.json"
    with file_path.open(mode="r", encoding="utf-8") as config_file:
        config_dict = json.load(config_file)
        if "database" in config_dict:
            return config_dict["database"]


def test_panelist_retrieve_all():
    """Testing for :py:meth:`wwdtm.panelist.Panelist.retrieve_all`."""
    panelist = Panelist(connect_dict=get_connect_dict())
    panelists = panelist.retrieve_all()

    assert panelists, "No panelists could be retrieved"
    assert "id" in panelists[0], "'id' was not returned for the first list item"
    assert "name" in panelists[0], "'name' was not returned for the first list item"
    assert "slug" in panelists[0], "'slug' was not returned for the first list item"
    assert "pronouns" in panelists[0], (
        "'pronouns' was not returned for the first list item"
    )


@pytest.mark.parametrize("use_decimal_scores", [True, False])
def test_panelist_retrieve_all_details(use_decimal_scores: bool):
    """Testing for :py:meth:`wwdtm.panelist.Panelist.retrieve_all_details`.

    :param use_decimal_scores: Flag set to use decimal score columns
        and values
    """
    panelist = Panelist(connect_dict=get_connect_dict())
    panelists = panelist.retrieve_all_details(use_decimal_scores=use_decimal_scores)

    assert panelists, "No panelists could be retrieved"
    assert "id" in panelists[0], "'id' was not returned for first list item"
    assert "name" in panelists[0], "'name' was not returned for the first list item"
    assert "slug" in panelists[0], "'slug' was not returned for the first list item"
    assert "pronouns" in panelists[0], (
        "'pronouns' was not returned for the first list item"
    )
    assert "is_host" in panelists[0], (
        "'is_host' was not returned for the first list item"
    )
    assert "appearances" in panelists[0], (
        "'appearances' was not returned for the first list item"
    )
    assert panelists[0]["appearances"], (
        "'appearances' does not contain data for the first list item"
    )
    assert "shows" in panelists[0]["appearances"], (
        "'shows' was not included in 'appearances' for ID the first list item"
    )
    assert "score_exception" in panelists[0]["appearances"]["shows"][0], (
        "'score_exception' was not included in first appearance for ID the first list item"
    )


def test_panelist_retrieve_all_ids():
    """Testing for :py:meth:`wwdtm.panelist.Panelist.retrieve_all_ids`."""
    panelist = Panelist(connect_dict=get_connect_dict())
    ids = panelist.retrieve_all_ids()

    assert ids, "No panelist IDs could be retrieved"


def test_panelist_retrieve_all_slugs():
    """Testing for :py:meth:`wwdtm.panelist.Panelist.retrieve_all_slugs`."""
    panelist = Panelist(connect_dict=get_connect_dict())
    slugs = panelist.retrieve_all_slugs()

    assert slugs, "No panelist slug strings could be retrieved"


@pytest.mark.parametrize("panelist_id", [13, 14])
def test_panelist_retrieve_by_id(panelist_id: int):
    """Testing for :py:meth:`wwdtm.panelist.Panelist.retrieve_by_id`.

    :param panelist_id: Panelist ID to test retrieving panelist
        information
    """
    panelist = Panelist(connect_dict=get_connect_dict())
    info = panelist.retrieve_by_id(panelist_id)

    assert info, f"Panelist ID {panelist_id} not found"
    assert "name" in info, f"'name' was not returned for ID {panelist_id}"
    assert "slug" in info, f"'slug' was not returned for ID {panelist_id}"
    assert "pronouns" in info, f"'pronouns' was not returned for ID {panelist_id}"


@pytest.mark.parametrize(
    "panelist_slug",
    ["drew-carey", "luke-burbank", "roxanne-roberts"],
)
def test_panelist_retrieve_by_slug(panelist_slug: str):
    """Testing for :py:meth:`wwdtm.panelist.Panelist.retrieve_by_slug`.

    :param panelist_slug: Panelist slug string to test retrieving
        panelist information
    """
    panelist = Panelist(connect_dict=get_connect_dict())
    info = panelist.retrieve_by_slug(panelist_slug)

    assert info, f"Panelist slug {panelist_slug} not found"
    assert "name" in info, f"'name' was not returned for slug {panelist_slug}"
    assert "slug" in info, f"'slug' was not returned for slug {panelist_slug}"
    assert "pronouns" in info, f"'pronouns' was not returned for slug {panelist_slug}"


@pytest.mark.parametrize(
    "panelist_id, use_decimal_scores",
    [(13, True), (13, False), (14, True), (14, False)],
)
def test_panelist_retrieve_details_by_id(panelist_id: int, use_decimal_scores: bool):
    """Testing for :py:meth:`wwdtm.panelist.Panelist.retrieve_details_by_id`.

    :param panelist_id: Panelist ID to test retrieving panelist details
    :param use_decimal_scores: Flag set to use decimal score columns
        and values
    """
    panelist = Panelist(connect_dict=get_connect_dict())
    info = panelist.retrieve_details_by_id(
        panelist_id, use_decimal_scores=use_decimal_scores
    )

    assert info, f"Panelist ID {panelist_id} not found"
    assert "name" in info, f"'name' was not returned for ID {panelist_id}"
    assert "slug" in info, f"'slug' was not returned for ID {panelist_id}"
    assert "pronouns" in info, f"'pronouns' was not returned for ID {panelist_id}"
    assert "is_host" in info, f"'is_host' was not returned for ID {panelist_id}"
    assert "appearances" in info, f"'appearances' was not returned for ID {panelist_id}"
    assert info["appearances"], (
        f"'appearances' does not contain data for ID {panelist_id}"
    )
    assert "shows" in info["appearances"], (
        f"'shows' was not included in 'appearances' for ID {panelist_id}"
    )
    assert "score_exception" in info["appearances"]["shows"][0], (
        f"'score_exception' was not included in first appearance for ID {panelist_id}"
    )


@pytest.mark.parametrize(
    "panelist_slug, use_decimal_scores",
    [
        ("luke-burbank", True),
        ("luke-burbank", False),
        ("roxanne-roberts", True),
        ("roxanne-roberts", False),
    ],
)
def test_panelist_retrieve_details_by_slug(
    panelist_slug: str, use_decimal_scores: bool
):
    """Testing for :py:meth:`wwdtm.panelist.Panelist.retrieve_details_by_slug`.

    :param panelist_slug: Panelist slug string to test retrieving
        panelist details
    :param use_decimal_scores: Flag set to use decimal score columns
        and values
    """
    panelist = Panelist(connect_dict=get_connect_dict())
    info = panelist.retrieve_details_by_slug(
        panelist_slug, use_decimal_scores=use_decimal_scores
    )

    assert info, f"Panelist slug {panelist_slug} not found"
    assert "name" in info, f"'name' was not returned for slug {panelist_slug}"
    assert "slug" in info, f"'slug' was not returned for slug {panelist_slug}"
    assert "pronouns" in info, f"'pronouns' was not returned for slug {panelist_slug}"
    assert "is_host" in info, f"'is_host' was not returned for slug {panelist_slug}"
    assert "appearances" in info, (
        f"'appearances' was not returned for slug {panelist_slug}"
    )
    assert info["appearances"], (
        f"'appearances' does not contain data for slug {panelist_slug}"
    )
    assert "shows" in info["appearances"], (
        f"'shows' was not included in 'appearances' for slug {panelist_slug}"
    )
    assert "score_exception" in info["appearances"]["shows"][0], (
        f"'score_exception' was not included in first appearance for slug {panelist_slug}"
    )


def test_panelist_retrieve_random_id() -> None:
    """Testing for :py:meth`wwdtm.panelist.Panelist.retrieve_random_id`."""
    panelist = Panelist(connect_dict=get_connect_dict())
    _id = panelist.retrieve_random_id()

    assert _id, "Returned random panelist ID is not valid"
    assert isinstance(_id, int), "Returned random panelist ID is not an integer"


def test_panelist_retrieve_random_slug() -> None:
    """Testing for :py:meth`wwdtm.panelist.Panelist.retrieve_random_slug`."""
    panelist = Panelist(connect_dict=get_connect_dict())
    _slug = panelist.retrieve_random_slug()

    assert _slug, "Returned random panelist slug string is not valid"
    assert isinstance(_slug, str), (
        "Returned random panelist slug string is not a string"
    )


def test_panelist_retrieve_random() -> None:
    """Testing for :py:meth:`wwdtm.panelist.Panelist.retrieve_random`."""
    panelist = Panelist(connect_dict=get_connect_dict())
    info = panelist.retrieve_random()

    assert info, "Random panelist not found"
    assert "name" in info, "'name' attribute was not returned for a random panelist"
    assert "slug" in info, "'slug' was not returned for a random panelist"
    assert "pronouns" in info, "'pronouns' was not returned for a random panelist"


@pytest.mark.parametrize("use_decimal_scores", [True, False])
def test_panelist_retrieve_random_details(use_decimal_scores: bool) -> None:
    """Testing for :py:meth:`wwdtm.panelist.Panelist.retrieve_random_details`."""
    panelist = Panelist(connect_dict=get_connect_dict())
    info = panelist.retrieve_random_details(use_decimal_scores=use_decimal_scores)

    assert info, "Random panelist not found"
    assert "name" in info, "'name' attribute was not returned for a random panelist"
    assert "slug" in info, "'slug' was not returned for a random panelist"
    assert "pronouns" in info, "'pronouns' was not returned for a random panelist"
    assert "appearances" in info, "'appearances' was not returned for a random panelist"



# Copyright (c) 2018-2026 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
# SPDX-License-Identifier: Apache-2.0
#
# vim: set noai syntax=python ts=4 sw=4:
import sys
from email.mime import base
from pathlib import Path

current_path = Path.cwd()
sys.path.insert(0, str(current_path.parent))

project = "wwdtm"
copyright = "2018–2026 Linh Pham. All Rights Reserved"
author = "Linh Pham"

extensions = [
    "sphinx.ext.napoleon",
    "sphinx_copybutton",
    "sphinx.ext.autodoc",
    "sphinx.ext.intersphinx",
    "sphinxext.opengraph",
]

templates_path = [
    "_templates",
]

exclude_patterns = [
    "_build",
    "Thumbs.db",
    ".DS_Store",
    ".git",
    ".venv",
    ".venv*",
    "venv",
    "venv*",
    "LICENSE",
    "DESCRIPTION",
]

source_suffix = [
    ".rst",
    ".md",
]

html_theme = "furo"

html_title = "Wait Wait Stats Library"

html_static_path = [
    "_static",
]

html_css_files = [
    "css/custom.css",
]

html_show_sourcelink = False
smartquotes = False

base_fonts = '"IBM Plex Sans", -apple-system, BlinkMacSystemFont, "Segoe UI", Helvetica, Arial, sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol";'
monospace_fonts = '"IBM Plex Mono", Menlo, Consolas, Monaco, "Liberation Mono", "Lucida Console", monospace;'

html_theme_options = {
    "light_css_variables": {
        "font-stack": base_fonts,
        "font-stack--monospace": monospace_fonts,
        "color-foreground-primary": "#000000",
        "color-foreground-secondary": "#161616",
        "color-foreground-muted": "#21272a",
        "color-brand-content": "#0043ce",
        "color-background-primary": "#ffffff",
        "color-sidebar-background": "#f2f4f8",
        "color-sidebar-link-text--top-level": "#0043ce",
        "color-api-name": "#520408",
        "color-api-pre-name": "#da1e28",
        "color-highlight-on-target": "#f2f4f8",
        "color-sidebar-search-foreground": "#000000",
        "sidebar-item-font-size": "1rem",
        "toc-font-size": "0.95rem",
        "toc-title-font-size": "1rem",
        "color-toc-item-text": "var(--color-foreground-primary);",
        "color-toc-title-text": "var(--color-foreground-primary);",
        "code-font-size": "1rem",
    },
    "dark_css_variables": {
        "font-stack": base_fonts,
        "font-stack--monospace": monospace_fonts,
        "color-foreground-primary": "#ffffff",
        "color-foreground-secondary": "#f4f4f4",
        "color-foreground-muted": "#dde1e6",
        "color-brand-content": "#78a9ff",
        "color-background-primary": "#161616",
        "color-sidebar-background": "#21272a",
        "color-sidebar-link-text--top-level": "#78a9ff",
        "color-api-name": "#fa4d56",
        "color-api-pre-name": "#ffb3b8",
        "color-highlight-on-target": "#21272a",
        "color-sidebar-search-foreground": "#ffffff",
        "sidebar-item-font-size": "1rem",
        "toc-font-size": "0.95rem",
        "toc-title-font-size": "1rem",
        "color-toc-item-text": "var(--color-foreground-primary);",
        "color-toc-title-text": "var(--color-foreground-primary);",
        "code-font-size": "1rem",
    },
}

pygments_style = "sas"
pygments_dark_style = "rrt"

intersphinx_mapping = {
    "python": ("https://docs.python.org/3", None),
}

autodoc_typehints = "description"
python_display_short_literal_types = True



# Copyright (c) 2018-2026 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
# SPDX-License-Identifier: Apache-2.0
#
# vim: set noai syntax=python ts=4 sw=4:
"""Wait Wait Stats Guest Appearance Retrieval Functions."""

from typing import Any

from mysql.connector import connect
from mysql.connector.connection import MySQLConnection
from mysql.connector.pooling import PooledMySQLConnection

from wwdtm.guest.utility import GuestUtility
from wwdtm.validation import valid_int_id


class GuestAppearances:
    """Guest appearance information retrieval class.

    Contains methods used to retrieve Not My Job guest appearance
    information, scores and scoring exceptions.

    :param connect_dict: A dictionary containing database connection
        settings as required by MySQL Connector/Python
    :param database_connection: MySQL database connection object
    """

    def __init__(
        self,
        connect_dict: dict[str, Any] = None,
        database_connection: MySQLConnection | PooledMySQLConnection = None,
    ):
        """Class initialization method."""
        if connect_dict:
            self.connect_dict = connect_dict
            self.database_connection = connect(**connect_dict)
        elif database_connection:
            if not database_connection.is_connected():
                database_connection.reconnect()

            self.database_connection = database_connection

        self.utility = GuestUtility(database_connection=self.database_connection)

    def retrieve_appearances_by_id(self, guest_id: int) -> dict[str, Any]:
        """Retrieves guest appearance information.

        :param guest_id: Guest ID
        :return: A dictionary containing guest appearances with
            corresponding show dates, Best Of or Repeat show flags,
            score and score exception flag
        """
        if not valid_int_id(guest_id):
            return {}

        query = """
            SELECT (
            SELECT COUNT(gm.showid) FROM ww_showguestmap gm
            JOIN ww_shows s ON s.showid = gm.showid
            WHERE s.bestof = 0 AND s.repeatshowid IS NULL AND
            gm.guestid = %s ) AS regular_shows, (
            SELECT COUNT(gm.showid) FROM ww_showguestmap gm
            JOIN ww_shows s ON s.showid = gm.showid
            WHERE gm.guestid = %s ) AS all_shows;
            """
        cursor = self.database_connection.cursor(dictionary=True)
        cursor.execute(
            query,
            (
                guest_id,
                guest_id,
            ),
        )
        result = cursor.fetchone()

        if result:
            appearance_counts = {
                "regular_shows": result["regular_shows"],
                "all_shows": result["all_shows"],
            }
        else:
            appearance_counts = {
                "regular_shows": 0,
                "all_shows": 0,
            }

        query = """
            SELECT gm.showid AS show_id, s.showdate AS date,
            s.bestof AS best_of, s.repeatshowid AS repeat_show_id,
            gm.guestscore AS score, gm.exception AS score_exception
            FROM ww_showguestmap gm
            JOIN ww_guests g ON g.guestid = gm.guestid
            JOIN ww_shows s ON s.showid = gm.showid
            WHERE gm.guestid = %s
            ORDER BY s.showdate ASC;
            """
        cursor = self.database_connection.cursor(dictionary=True)
        cursor.execute(query, (guest_id,))
        results = cursor.fetchall()
        cursor.close()

        if results:
            appearances = []
            for appearance in results:
                info = {
                    "show_id": appearance["show_id"],
                    "date": appearance["date"].isoformat(),
                    "best_of": bool(appearance["best_of"]),
                    "repeat_show": bool(appearance["repeat_show_id"]),
                    "score": appearance["score"],
                    "score_exception": bool(appearance["score_exception"]),
                }
                appearances.append(info)

            return {
                "count": appearance_counts,
                "shows": appearances,
            }
        else:
            return {
                "count": appearance_counts,
                "shows": [],
            }

    def retrieve_appearances_by_slug(self, guest_slug: str) -> dict[str, Any]:
        """Retrieves guest appearance information.

        :param guest_slug: Guest slug string
        :return: A dictionary containing guest appearances with
            corresponding show dates, Best Of or Repeat show flags,
            score and score exception flag
        """
        id_ = self.utility.convert_slug_to_id(guest_slug)
        if not id_:
            return {}

        return self.retrieve_appearances_by_id(id_)



# Copyright (c) 2018-2026 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
# SPDX-License-Identifier: Apache-2.0
#
# vim: set noai syntax=python ts=4 sw=4:
"""Testing for object: :py:module:`wwdtm`."""

import json
from pathlib import Path
from typing import Any

import pytest

from wwdtm import database_version


@pytest.mark.skip
def get_connect_dict() -> dict[str, Any]:
    """Retrieves database connection settings.

    :return: A dictionary containing database connection
        settings as required by MySQL Connector/Python
    """
    file_path = Path.cwd() / "config.json"
    with file_path.open(mode="r", encoding="utf-8") as config_file:
        config_dict = json.load(config_file)
        if "database" in config_dict:
            return config_dict["database"]

    return None


def test_database_version():
    """Testing for :py:func:`wwdtm.database_version`."""
    _database_version = database_version(connect_dict=get_connect_dict())

    assert isinstance(_database_version, tuple), (
        "Invalid database version value type returned"
    )

    if isinstance(_database_version, tuple):
        assert len(_database_version) == 2 or len(_database_version) == 3, (
            "Invalid database version value returned"
        )



# Copyright (c) 2018-2026 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
# SPDX-License-Identifier: Apache-2.0
#
# vim: set noai syntax=python ts=4 sw=4:
"""Testing for object: :py:class:`wwdtm.pronoun.Pronouns`."""

import json
from pathlib import Path
from typing import Any

import pytest

from wwdtm.pronoun import Pronouns


@pytest.mark.skip
def get_connect_dict() -> dict[str, Any]:
    """Retrieves database connection settings.

    :return: A dictionary containing database connection
        settings as required by MySQL Connector/Python
    """
    file_path = Path.cwd() / "config.json"
    with file_path.open(mode="r", encoding="utf-8") as config_file:
        config_dict = json.load(config_file)
        if "database" in config_dict:
            return config_dict["database"]


def test_pronouns_retrieve_all():
    """Testing for :py:meth:`wwdtm.pronoun.Pronouns.retrieve_all`."""
    pn = Pronouns(connect_dict=get_connect_dict())
    all_pronouns = pn.retrieve_all()

    assert all_pronouns, "No pronouns could be retrieved"
    assert "id" in all_pronouns[0], "'id' was not returned for the first list item"
    assert "pronouns" in all_pronouns[0], (
        "'pronouns' was not returned for the first list item"
    )


def test_pronouns_retrieve_all_ids():
    """Testing for :py:meth:`wwdtm.pronoun.Pronouns.retrieve_all_ids`."""
    pn = Pronouns(connect_dict=get_connect_dict())
    ids = pn.retrieve_all_ids()

    assert ids, "No pronouns IDs could be retrieved"


def test_pronouns_retrieve_all_as_dict():
    """Testing for :py:meth:`wwdtm.pronoun.Pronouns.retrieve_all_as_dict`."""
    pn = Pronouns(connect_dict=get_connect_dict())
    all_pronouns = pn.retrieve_all_as_dict()

    assert all_pronouns, "No pronouns could be retrieved"


def test_pronouns_retrieve_all_pronouns():
    """Testing for :py:meth:`wwdtm.pronoun.Pronouns.retrieve_all_pronouns`."""
    pn = Pronouns(connect_dict=get_connect_dict())
    all_pronouns = pn.retrieve_all_pronouns()

    assert all_pronouns, "No pronouns strings could be retrieved"


@pytest.mark.parametrize("pronouns_id", [1])
def test_pronouns_retrieve_by_id(pronouns_id: int):
    """Testing for :py:meth:`wwdtm.pronoun.Pronouns.retrieve_by_id`.

    :param pronouns_id: Pronouns ID to test retrieving pronouns
        information
    """
    pn = Pronouns(connect_dict=get_connect_dict())
    info = pn.retrieve_by_id(pronouns_id)

    assert info, f"Pronouns ID {pronouns_id} not found"
    assert "id" in info, f"'id' was not returned for ID {pronouns_id}"
    assert "pronouns" in info, f"'pronouns' was not returned for ID {pronouns_id}"



# Copyright (c) 2018-2026 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
# SPDX-License-Identifier: Apache-2.0
#
# vim: set noai syntax=python ts=4 sw=4:
"""Testing for object: :py:class:`wwdtm.scorekeeper.ScorekeeperAppearances`."""

import json
from pathlib import Path
from typing import Any

import pytest

from wwdtm.scorekeeper import ScorekeeperAppearances


@pytest.mark.skip
def get_connect_dict() -> dict[str, Any]:
    """Retrieves database connection settings.

    :return: A dictionary containing database connection
        settings as required by MySQL Connector/Python
    """
    file_path = Path.cwd() / "config.json"
    with file_path.open(mode="r", encoding="utf-8") as config_file:
        config_dict = json.load(config_file)
        if "database" in config_dict:
            return config_dict["database"]


@pytest.mark.parametrize("scorekeeper_id", [13])
def test_scorekeeper_appearance_retrieve_appearances_by_id(scorekeeper_id: int):
    """Testing for :py:meth:`wwdtm.scorekeeper.ScorekeeperAppearances.retrieve_appearances_by_id`.

    :param scorekeeper_id: Scorekeeper ID to test retrieving scorekeeper
        appearances
    """
    appearances = ScorekeeperAppearances(connect_dict=get_connect_dict())
    appearance = appearances.retrieve_appearances_by_id(scorekeeper_id)

    assert "count" in appearance, f"'count' was not returned for ID {scorekeeper_id}"
    assert "shows" in appearance, f"'shows' was not returned for ID {scorekeeper_id}"


@pytest.mark.parametrize("scorekeeper_slug", ["chioke-i-anson"])
def test_scorekeeper_appearance_retrieve_appearances_by_slug(scorekeeper_slug: str):
    """Testing for :py:meth:`wwdtm.scorekeeper.ScorekeeperAppearances.retrieve_appearances_by_slug`.

    :param scorekeeper_slug: Scorekeeper slug string to test retrieving
        scorekeeper appearances
    """
    appearances = ScorekeeperAppearances(connect_dict=get_connect_dict())
    appearance = appearances.retrieve_appearances_by_slug(scorekeeper_slug)

    assert "count" in appearance, (
        f"'count' was not returned for slug {scorekeeper_slug}"
    )
    assert "shows" in appearance, (
        f"'shows' was not returned for slug {scorekeeper_slug}"
    )



# Copyright (c) 2018-2026 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
# SPDX-License-Identifier: Apache-2.0
#
# vim: set noai syntax=python ts=4 sw=4:
"""Wait Wait Stats Location Recordings Retrieval Functions."""

from typing import Any

from mysql.connector import connect
from mysql.connector.connection import MySQLConnection
from mysql.connector.pooling import PooledMySQLConnection

from wwdtm.location.utility import LocationUtility
from wwdtm.validation import valid_int_id


class LocationRecordings:
    """Location recording information retrieval class.

    Contains methods used to retrieve recording information, including
    show flags.

    :param connect_dict: A dictionary containing database connection
        settings as required by MySQL Connector/Python
    :param database_connection: MySQL database connection object
    """

    def __init__(
        self,
        connect_dict: dict[str, Any] = None,
        database_connection: MySQLConnection | PooledMySQLConnection = None,
    ):
        """Class initialization method."""
        if connect_dict:
            self.connect_dict = connect_dict
            self.database_connection = connect(**connect_dict)
        elif database_connection:
            if not database_connection.is_connected():
                database_connection.reconnect()

            self.database_connection = database_connection

        self.utility = LocationUtility(database_connection=self.database_connection)

    def retrieve_recordings_by_id(self, location_id: int) -> dict[str, Any]:
        """Retrieves location recording information.

        Location recording information includes the corresponding show
        dates and Best Of or Repeat show flags.

        :param location_id: Location ID
        :return: A dictionary containing location recording counts,
            dates and show flags
        """
        if not valid_int_id(location_id):
            return {}

        query = """
            SELECT (
            SELECT COUNT(lm.showid) FROM ww_showlocationmap lm
            JOIN ww_shows s ON s.showid = lm.showid
            WHERE s.bestof = 0 AND s.repeatshowid IS NULL AND
            lm.locationid = %s ) AS regular_shows, (
            SELECT COUNT(lm.showid) FROM ww_showlocationmap lm
            JOIN ww_shows s ON s.showid = lm.showid
            WHERE lm.locationid = %s ) AS all_shows;
            """
        cursor = self.database_connection.cursor(dictionary=True)
        cursor.execute(
            query,
            (
                location_id,
                location_id,
            ),
        )
        result = cursor.fetchone()

        recording_counts = {
            "regular_shows": result["regular_shows"],
            "all_shows": result["all_shows"],
        }

        query = """
            SELECT lm.showid AS show_id, s.showdate AS date,
            s.bestof AS best_of, s.repeatshowid AS repeat_show_id
            FROM ww_showlocationmap lm
            JOIN ww_shows s ON s.showid = lm.showid
            WHERE lm.locationid = %s
            ORDER BY s.showdate ASC;
            """
        cursor.execute(query, (location_id,))
        results = cursor.fetchall()
        cursor.close()

        if results:
            recordings = []
            for recording in results:
                info = {
                    "show_id": recording["show_id"],
                    "date": recording["date"].isoformat(),
                    "best_of": bool(recording["best_of"]),
                    "repeat_show": bool(recording["repeat_show_id"]),
                }
                recordings.append(info)

            return {
                "count": recording_counts,
                "shows": recordings,
            }
        else:
            return {
                "count": recording_counts,
                "shows": [],
            }

    def retrieve_recordings_by_slug(self, location_slug: str) -> dict[str, Any]:
        """Retrieves location recording information.

        Location recording information includes the corresponding show
        dates and Best Of or Repeat show flags.

        :param location_slug: Location slug string
        :return: A dictionary containing location recording counts,
            dates and show flags
        """
        id_ = self.utility.convert_slug_to_id(location_slug)
        if not id_:
            return {}

        return self.retrieve_recordings_by_id(id_)



# Copyright (c) 2018-2026 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
# SPDX-License-Identifier: Apache-2.0
#
# vim: set noai syntax=python ts=4 sw=4:
"""Wait Wait Stats: Panelist module."""

from wwdtm.panelist.appearances import PanelistAppearances
from wwdtm.panelist.decimal_scores import PanelistDecimalScores
from wwdtm.panelist.panelist import Panelist
from wwdtm.panelist.scores import PanelistScores
from wwdtm.panelist.statistics import PanelistStatistics
from wwdtm.panelist.utility import PanelistUtility



# Copyright (c) 2018-2026 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
# SPDX-License-Identifier: Apache-2.0
#
# vim: set noai syntax=python ts=4 sw=4:
"""Wait Wait Stats Location Data Retrieval Functions."""

from typing import Any

from mysql.connector import connect
from mysql.connector.connection import MySQLConnection
from mysql.connector.pooling import PooledMySQLConnection

from wwdtm.location.recordings import LocationRecordings
from wwdtm.location.utility import LocationUtility
from wwdtm.validation import valid_int_id


class Location:
    """Location information retrieval class.

    Contains methods used to retrieve location IDs, venue names and
    locations, including city and state, and recording dates.

    :param connect_dict: A dictionary containing database connection
        settings as required by MySQL Connector/Python
    :param database_connection: MySQL database connection object
    """

    def __init__(
        self,
        connect_dict: dict[str, Any] = None,
        database_connection: MySQLConnection | PooledMySQLConnection = None,
    ):
        """Class initialization method."""
        if connect_dict:
            self.connect_dict = connect_dict
            self.database_connection = connect(**connect_dict)
        elif database_connection:
            if not database_connection.is_connected():
                database_connection.reconnect()

            self.database_connection = database_connection

        self.recordings = LocationRecordings(
            database_connection=self.database_connection
        )
        self.utility = LocationUtility(database_connection=self.database_connection)

    def retrieve_all(self, sort_by_venue: bool = False) -> list[dict[str, int | str]]:
        """Retrieves location information for all locations.

        :param sort_by_venue: A boolean to determine if the returned
            list is sorted by venue first or by state and city first
        :return: A list of dictionaries containing location ID, venue
            name, city, state and slug string
        """
        query = """
            SELECT l.locationid AS id, l.city, l.state,
            pa.name AS state_name, l.venue, l.latitude, l.longitude,
            l.locationslug AS slug
            FROM ww_locations l
            LEFT JOIN ww_postal_abbreviations pa ON pa.postal_abbreviation = l.state
            """

        if sort_by_venue:
            query = query + "ORDER BY l.venue ASC, l.city ASC, pa.name ASC;"
        else:
            query = query + "ORDER BY pa.name ASC, l.city ASC, l.venue ASC;"

        cursor = self.database_connection.cursor(dictionary=True)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        locations = []
        for row in results:
            if not row["latitude"] and not row["longitude"]:
                coordinates = None
            else:
                coordinates = {
                    "latitude": row["latitude"] if row["latitude"] else None,
                    "longitude": row["longitude"] if row["longitude"] else None,
                }

            locations.append(
                {
                    "id": row["id"],
                    "city": row["city"],
                    "state": row["state"],
                    "state_name": row["state_name"],
                    "venue": row["venue"],
                    "coordinates": coordinates,
                    "slug": (
                        row["slug"]
                        if row["slug"]
                        else self.utility.slugify_location(
                            location_id=row["id"],
                            venue=row["venue"],
                            city=row["city"],
                            state=row["state"],
                        )
                    ),
                }
            )

        return locations

    def retrieve_all_details(self, sort_by_venue: bool = False) -> list[dict[str, Any]]:
        """Retrieves location information and recordings for all locations.

        :param sort_by_venue: A boolean to determine if the returned
            list is sorted by venue first or by state and city first
        :return: A list of dictionaries containing location ID, venue
            name, city, state, slug string and a list of recordings
        """
        query = """
            SELECT l.locationid AS id, l.city, l.state,
            pa.name AS state_name, l.venue, l.latitude, l.longitude,
            l.locationslug AS slug
            FROM ww_locations l
            LEFT JOIN ww_postal_abbreviations pa ON pa.postal_abbreviation = l.state
            """
        if sort_by_venue:
            query = query + " ORDER BY l.venue ASC, l.city ASC, pa.name ASC;"
        else:
            query = query + " ORDER BY pa.name ASC, l.city ASC, l.venue ASC;"

        cursor = self.database_connection.cursor(dictionary=True)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        locations = []
        for row in results:
            if not row["latitude"] and not row["longitude"]:
                coordinates = None
            else:
                coordinates = {
                    "latitude": row["latitude"] if row["latitude"] else None,
                    "longitude": row["longitude"] if row["longitude"] else None,
                }

            locations.append(
                {
                    "id": row["id"],
                    "city": row["city"],
                    "state": row["state"],
                    "state_name": row["state_name"],
                    "venue": row["venue"],
                    "coordinates": coordinates,
                    "slug": (
                        row["slug"]
                        if row["slug"]
                        else self.utility.slugify_location(
                            location_id=row["id"],
                            venue=row["venue"],
                            city=row["city"],
                            state=row["state"],
                        )
                    ),
                    "recordings": self.recordings.retrieve_recordings_by_id(row["id"]),
                }
            )

        return locations

    def retrieve_all_ids(self, sort_by_venue: bool = False) -> list[int]:
        """Retrieves all location IDs.

        :param sort_by_venue: A boolean to determine if the returned
            list is sorted by venue first or by state and city first
        :return: A list of location IDs as integers
        """
        query = """
            SELECT l.locationid
            FROM ww_locations l
            LEFT JOIN ww_postal_abbreviations pa ON pa.postal_abbreviation = l.state
            """
        if sort_by_venue:
            query = query + " ORDER BY l.venue ASC, l.city ASC, pa.name ASC;"
        else:
            query = query + " ORDER BY pa.name ASC, l.city ASC, l.venue ASC;"

        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        return [v[0] for v in results]

    def retrieve_all_slugs(self, sort_by_venue: bool = False) -> list[str]:
        """Retrieves all location slug strings.

        :param sort_by_venue: A boolean to determine if the returned
            list is sorted by venue first or by state and city first
        :return: A list of location slug strings
        """
        query = """
            SELECT l.locationslug
            FROM ww_locations l
            LEFT JOIN ww_postal_abbreviations pa ON pa.postal_abbreviation = l.state
            """
        if sort_by_venue:
            query = query + " ORDER BY l.venue ASC, l.city ASC, pa.name ASC;"
        else:
            query = query + " ORDER BY pa.name ASC, l.city ASC, l.venue ASC;"

        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        return [v[0] for v in results]

    def retrieve_by_id(self, location_id: int) -> dict[str, int | str]:
        """Retrieves location information.

        :param location_id: Location ID
        :return: A dictionary containing location ID, venue, city, state
            and slug string
        """
        if not valid_int_id(location_id):
            return {}

        query = """
            SELECT l.locationid AS id, l.city, l.state,
            pa.name AS state_name, l.venue, l.latitude, l.longitude,
            l.locationslug AS slug
            FROM ww_locations l
            LEFT JOIN ww_postal_abbreviations pa ON pa.postal_abbreviation = l.state
            WHERE l.locationid = %s
            LIMIT 1;
            """
        cursor = self.database_connection.cursor(dictionary=True)
        cursor.execute(query, (location_id,))
        result = cursor.fetchone()
        cursor.close()

        if not result:
            return {}

        if not result["latitude"] and not result["longitude"]:
            coordinates = None
        else:
            coordinates = {
                "latitude": result["latitude"] if result["latitude"] else None,
                "longitude": result["longitude"] if result["longitude"] else None,
            }

        return {
            "id": result["id"],
            "city": result["city"],
            "state": result["state"],
            "state_name": result["state_name"],
            "venue": result["venue"],
            "coordinates": coordinates,
            "slug": (
                result["slug"]
                if result["slug"]
                else self.utility.slugify_location(
                    location_id=result["id"],
                    venue=result["venue"],
                    city=result["city"],
                    state=result["state"],
                )
            ),
        }

    def retrieve_by_slug(self, location_slug: str) -> dict[str, int | str]:
        """Retrieves location information.

        :param location_slug: Location slug string
        :return: A dictionary containing location ID, venue, city, state
            and slug string
        """
        try:
            slug = location_slug.strip()
            if not slug:
                return {}
        except AttributeError:
            return {}

        id_ = self.utility.convert_slug_to_id(slug)
        if not id_:
            return {}

        return self.retrieve_by_id(id_)

    def retrieve_details_by_id(self, location_id: int) -> dict[str, Any]:
        """Retrieves location information and recordings.

        :param location_id: Location ID
        :return: A dictionary containing location ID, venue name, city,
            state, slug string and a list of recordings
        """
        if not valid_int_id(location_id):
            return {}

        info = self.retrieve_by_id(location_id)
        if not info:
            return {}

        info["recordings"] = self.recordings.retrieve_recordings_by_id(location_id)

        return info

    def retrieve_details_by_slug(self, location_slug: str) -> dict[str, Any]:
        """Retrieves location information and recordings.

        :param location_slug: Location slug string
        :return: A dictionary containing location ID, venue name, city,
            state, slug string and a list of recordings
        """
        try:
            slug = location_slug.strip()
            if not slug:
                return {}
        except AttributeError:
            return {}

        id_ = self.utility.convert_slug_to_id(slug)
        if not id_:
            return {}

        return self.retrieve_details_by_id(id_)

    def retrieve_postal_abbreviations(self) -> dict[str, dict[str, str]]:
        """Retrieves postal abbreviations, corresponding names and countries.

        :return: A dictionary containing postal abbreviation as keys
            and corresponding name and country as values.
        """
        query = """
            SELECT postal_abbreviation, name, country
            FROM ww_postal_abbreviations
            ORDER BY postal_abbreviation ASC;
            """
        cursor = self.database_connection.cursor(dictionary=True)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return None

        abbreviations = {}
        for row in results:
            abbreviations[row["postal_abbreviation"]] = {
                "name": row["name"],
                "country": row["country"],
            }

        return abbreviations

    def retrieve_postal_abbreviations_list(self) -> list[dict[str, str]]:
        """Retrieves postal abbreviations, corresponding name and countries as a list.

        :return: A list of dictionaries, each containing postal abbreviation,
            corresponding name and country.
        """
        _postal_abbreviations = self.retrieve_postal_abbreviations()

        if not _postal_abbreviations:
            return None

        abbreviations = []
        for abbreviation in _postal_abbreviations:
            abbreviations.append(
                {
                    "postal_abbreviation": abbreviation,
                    "name": _postal_abbreviations[abbreviation]["name"],
                    "country": _postal_abbreviations[abbreviation]["country"],
                }
            )

        return abbreviations

    def retrieve_postal_details_by_abbreviation(
        self,
        abbreviation: str,
    ) -> dict[str, str]:
        """Retrieves postal abbreviation information for a specific abbreviation.

        :param abbreviation: Postal Abbreviation
        :return: A dictionary containing postal abbreviation,
            corresponding name, and country.
        """
        if not abbreviation:
            return None

        query = """
            SELECT postal_abbreviation, name, country
            FROM ww_postal_abbreviations
            WHERE postal_abbreviation = %s
            ORDER BY postal_abbreviation ASC;
            """
        cursor = self.database_connection.cursor(dictionary=True)
        cursor.execute(query, (abbreviation,))
        result = cursor.fetchone()
        cursor.close()

        if not result:
            return None

        return {
            "postal_abbreviation": result["postal_abbreviation"],
            "name": result["name"],
            "country": result["country"],
        }

    def retrieve_random_id(self) -> int:
        """Retrieves an ID for a random location.

        :return: ID for a random location.
        """
        query = """
            SELECT locationid FROM ww_locations
            WHERE locationslug <> 'tbd'
            ORDER BY RAND()
            LIMIT 1;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query)
        result = cursor.fetchone()
        cursor.close()

        if not result:
            return None

        return result[0]

    def retrieve_random_slug(self) -> str:
        """Retrieves a slug string for a random location.

        :return: Slug string for a random location.
        """
        query = """
            SELECT locationslug FROM ww_locations
            WHERE locationslug <> 'tbd'
            ORDER BY RAND()
            LIMIT 1;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query)
        result = cursor.fetchone()
        cursor.close()

        if not result:
            return None

        return result[0]

    def retrieve_random(self) -> dict[str, int | str]:
        """Retrieves information for a random location.

        :return: A dictionary containing location ID, venue, city, state
            and slug string
        """
        _id = self.retrieve_random_id()

        if not _id:
            return None

        return self.retrieve_by_id(location_id=_id)

    def retrieve_random_details(self) -> dict[str, Any]:
        """Retrieves information and recordings for a random location.

        :return: A dictionary containing location ID, venue name, city,
            state, slug string and a list of recordings
        """
        _id = self.retrieve_random_id()

        if not _id:
            return None

        return self.retrieve_details_by_id(location_id=_id)



# Copyright (c) 2018-2026 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
# SPDX-License-Identifier: Apache-2.0
#
# vim: set noai syntax=python ts=4 sw=4:
"""Wait Wait Stats Scorekeeper Appearance Retrieval Functions."""

from typing import Any

from mysql.connector import connect
from mysql.connector.connection import MySQLConnection
from mysql.connector.pooling import PooledMySQLConnection

from wwdtm.scorekeeper.utility import ScorekeeperUtility
from wwdtm.validation import valid_int_id


class ScorekeeperAppearances:
    """Scorekeeper appearance information retrieval class.

    Contains methods used to retrieve scorekeeper appearance
    information, including show flags.

    :param connect_dict: A dictionary containing database connection
        settings as required by MySQL Connector/Python
    :param database_connection: MySQL database connection object
    """

    def __init__(
        self,
        connect_dict: dict[str, Any] = None,
        database_connection: MySQLConnection | PooledMySQLConnection = None,
    ):
        """Class initialization method."""
        if connect_dict:
            self.connect_dict = connect_dict
            self.database_connection = connect(**connect_dict)
        elif database_connection:
            if not database_connection.is_connected():
                database_connection.reconnect()

            self.database_connection = database_connection

        self.utility = ScorekeeperUtility(database_connection=self.database_connection)

    def retrieve_appearances_by_id(self, scorekeeper_id: int) -> dict[str, Any]:
        """Retrieves scorekeeper appearance information.

        :param scorekeeper_id: Scorekeeper ID
        :return: A dictionary containing scorekeeper appearances with
            corresponding show dates and Best Of or Repeat show flags
        """
        if not valid_int_id(scorekeeper_id):
            return {}

        query = """
            SELECT (
            SELECT COUNT(skm.showid) FROM ww_showskmap skm
            JOIN ww_shows s ON s.showid = skm.showid
            WHERE s.bestof = 0 AND s.repeatshowid IS NULL AND
            skm.scorekeeperid = %s ) AS regular_shows, (
            SELECT COUNT(skm.showid) FROM ww_showskmap skm
            JOIN ww_shows s ON s.showid = skm.showid
            WHERE skm.scorekeeperid = %s ) AS all_shows;
            """
        cursor = self.database_connection.cursor(dictionary=True)
        cursor.execute(
            query,
            (
                scorekeeper_id,
                scorekeeper_id,
            ),
        )
        result = cursor.fetchone()

        if result:
            appearance_counts = {
                "regular_shows": result["regular_shows"],
                "all_shows": result["all_shows"],
            }
        else:
            appearance_counts = {
                "regular_shows": 0,
                "all_shows": 0,
            }

        query = """
            SELECT skm.showid AS show_id, s.showdate AS date,
            s.bestof AS best_of, s.repeatshowid AS repeat_show_id,
            skm.guest, skm.description
            FROM ww_showskmap skm
            JOIN ww_scorekeepers sk ON sk.scorekeeperid = skm.scorekeeperid
            JOIN ww_shows s ON s.showid = skm.showid
            WHERE sk.scorekeeperid = %s
            ORDER BY s.showdate ASC;
            """
        cursor.execute(query, (scorekeeper_id,))
        results = cursor.fetchall()
        cursor.close()

        if results:
            appearances = []
            for appearance in results:
                info = {
                    "show_id": appearance["show_id"],
                    "date": appearance["date"].isoformat(),
                    "best_of": bool(appearance["best_of"]),
                    "repeat_show": bool(appearance["repeat_show_id"]),
                    "guest": bool(appearance["guest"]),
                    "description": appearance["description"],
                }
                appearances.append(info)

            return {
                "count": appearance_counts,
                "shows": appearances,
            }

        else:
            return {
                "count": appearance_counts,
                "shows": [],
            }

    def retrieve_appearances_by_slug(self, scorekeeper_slug: str) -> dict[str, Any]:
        """Retrieves scorekeeper appearance information.

        :param scorekeeper_slug: Scorekeeper slug string
        :return: A dictionary containing scorekeeper appearances with
            corresponding show dates and Best Of or Repeat show flags
        """
        id_ = self.utility.convert_slug_to_id(scorekeeper_slug)
        if not id_:
            return {}

        return self.retrieve_appearances_by_id(id_)



# Copyright (c) 2018-2026 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
# SPDX-License-Identifier: Apache-2.0
#
# vim: set noai syntax=python ts=4 sw=4:
"""Wait Wait Stats Show Data Utility Functions."""

import datetime
from typing import Any

from mysql.connector import connect
from mysql.connector.connection import MySQLConnection
from mysql.connector.pooling import PooledMySQLConnection

from wwdtm.validation import valid_int_id


class ShowUtility:
    """Show information and utility class.

    Contains methods to convert between show ID and date, and to check
    if show IDs and dates exist.

    :param connect_dict: A dictionary containing database connection
        settings as required by MySQL Connector/Python
    :param database_connection: MySQL database connection object
    """

    def __init__(
        self,
        connect_dict: dict[str, Any] = None,
        database_connection: MySQLConnection | PooledMySQLConnection = None,
    ):
        """Class initialization method."""
        if connect_dict:
            self.connect_dict = connect_dict
            self.database_connection = connect(**connect_dict)
        elif database_connection:
            if not database_connection.is_connected():
                database_connection.reconnect()

            self.database_connection = database_connection

    def convert_date_to_id(self, year: int, month: int, day: int) -> int | None:
        """Converts a show date to the corresponding show ID.

        :param year: Year portion of a show date
        :param month: Month portion of a show date
        :param day: Day portion of a show date
        :return: Show ID if a corresponding value is found. Otherwise,
            ``None`` is returned
        """
        try:
            show_date = datetime.datetime(year, month, day)
        except ValueError:
            return None

        query = """
            SELECT showid FROM ww_shows WHERE showdate = %s LIMIT 1;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query, (show_date.isoformat(),))
        result = cursor.fetchone()
        cursor.close()

        if result:
            return result[0]

        return None

    def convert_id_to_date(self, show_id: int) -> str | None:
        """Converts a show ID to the corresponding show date.

        :param show_id: Show ID
        :return: Show date if a corresponding value is found. Otherwise,
            ``None`` is returned
        """
        if not valid_int_id(show_id):
            return None

        query = """
            SELECT showdate FROM ww_shows WHERE showid = %s LIMIT 1;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query, (show_id,))
        result = cursor.fetchone()
        cursor.close()

        if result:
            return result[0].isoformat()

        return None

    def date_exists(self, year: int, month: int, day: int) -> bool:
        """Validates if a show date exists.

        :param year: Year portion of a show date
        :param month: Month portion of a show date
        :param day: Day portion of a show date
        :return: ``True`` if the show date exists, otherwise ``False``
        """
        try:
            show_date = datetime.datetime(year, month, day)
        except ValueError:
            return False

        query = """
            SELECT showid FROM ww_shows WHERE showdate = %s LIMIT 1;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query, (show_date.isoformat(),))
        result = cursor.fetchone()
        cursor.close()

        return bool(result)

    def id_exists(self, show_id: int) -> bool:
        """Validates if a show ID exists.

        :param show_id: Show ID
        :return: ``True`` if the show ID exists, otherwise ``False``
        """
        if not valid_int_id(show_id):
            return False

        query = """
            SELECT showid FROM ww_shows WHERE showid = %s LIMIT 1;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query, (show_id,))
        result = cursor.fetchone()
        cursor.close()

        return bool(result)



# Copyright (c) 2018-2026 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
# SPDX-License-Identifier: Apache-2.0
#
# vim: set noai syntax=python ts=4 sw=4:
"""Testing for object: :py:class:`wwdtm.location.Location`."""

import json
from pathlib import Path
from typing import Any

import pytest
from numpy import isin

from wwdtm.location import Location


@pytest.mark.skip
def get_connect_dict() -> dict[str, Any]:
    """Retrieves database connection settings.

    :return: A dictionary containing database connection
        settings as required by MySQL Connector/Python
    """
    file_path = Path.cwd() / "config.json"
    with file_path.open(mode="r", encoding="utf-8") as config_file:
        config_dict = json.load(config_file)
        if "database" in config_dict:
            return config_dict["database"]


def test_location_retrieve_all():
    """Testing for :py:meth:`wwdtm.location.Location.retrieve_all`."""
    location = Location(connect_dict=get_connect_dict())
    locations = location.retrieve_all()

    assert locations, "No locations could be retrieved"
    assert "id" in locations[0], "'id' was not returned for the first list item"
    assert "venue" in locations[0], "'venue' was not returned for the first list item"
    assert "coordinates" in locations[0], (
        "'coordinates' was not returned for the first list item"
    )
    if locations[0]["coordinates"]:
        assert "latitude" in locations[0]["coordinates"], (
            "'latitude' was not returned for the first list item"
        )
        assert "longitude" in locations[0]["coordinates"], (
            "'longitude' was not returned for the first list item"
        )


def test_location_retrieve_all_details():
    """Testing for :py:meth:`wwdtm.location.Location.retrieve_all_details`."""
    location = Location(connect_dict=get_connect_dict())
    locations = location.retrieve_all_details()

    assert locations, "No locations could be retrieved"
    assert "id" in locations[0], "'id' was not returned for first list item"
    assert "venue" in locations[0], "'venue' was not returned for the first list item"
    assert "coordinates" in locations[0], (
        "'coordinates' was not returned for the first list item"
    )
    if locations[0]["coordinates"]:
        assert "latitude" in locations[0]["coordinates"], (
            "'latitude' was not returned for the first list item"
        )
        assert "longitude" in locations[0]["coordinates"], (
            "'longitude' was not returned for the first list item"
        )
    assert "recordings" in locations[0], (
        "'recordings' was not returned for the first list item"
    )


def test_location_retrieve_all_ids():
    """Testing for :py:meth:`wwdtm.location.Location.retrieve_all_ids`."""
    location = Location(connect_dict=get_connect_dict())
    ids = location.retrieve_all_ids()

    assert ids, "No location IDs could be retrieved"


def test_location_retrieve_all_slugs():
    """Testing for :py:meth:`wwdtm.location.Location.retrieve_all_slugs`."""
    location = Location(connect_dict=get_connect_dict())
    slugs = location.retrieve_all_slugs()

    assert slugs, "No location slug strings could be retrieved"


@pytest.mark.parametrize("location_id", [95, 148])
def test_location_retrieve_by_id(location_id: int):
    """Testing for :py:meth:`wwdtm.location.Location.retrieve_by_id`.

    :param location_id: Location ID to test retrieving location
        information
    """
    location = Location(connect_dict=get_connect_dict())
    info = location.retrieve_by_id(location_id)

    assert info, f"Location ID {location_id} not found"
    assert "venue" in info, f"'venue' was not returned for ID {location_id}"
    assert "coordinates" in info, f"'coordinates' was not returned for ID {location_id}"
    if info["coordinates"]:
        assert "latitude" in info["coordinates"], (
            f"'latitude' was not returned for ID {location_id}"
        )
        assert "longitude" in info["coordinates"], (
            f"'longitude' was not returned for ID {location_id}"
        )


@pytest.mark.parametrize("location_id", [95, 148])
def test_location_retrieve_details_by_id(location_id: int):
    """Testing for :py:meth:`wwdtm.location.location.retrieve_details_by_id`.

    :param location_id: Location ID to test retrieving location details
    """
    location = Location(connect_dict=get_connect_dict())
    info = location.retrieve_details_by_id(location_id)

    assert info, f"Location ID {location_id} not found"
    assert "venue" in info, f"'venue' was not returned for ID {location_id}"
    assert "coordinates" in info, f"'coordinates' was not returned for ID {location_id}"
    if info["coordinates"]:
        assert "latitude" in info["coordinates"], (
            f"'latitude' was not returned for ID {location_id}"
        )
        assert "longitude" in info["coordinates"], (
            f"'longitude' was not returned for ID {location_id}"
        )
    assert "recordings" in info, f"'recordings' was not returned for ID {location_id}"


@pytest.mark.parametrize("location_slug", ["the-chicago-theatre-chicago-il"])
def test_location_retrieve_by_slug(location_slug: str):
    """Testing for :py:meth:`wwdtm.location.Location.retrieve_by_slug`.

    :param location_slug: Location slug string to test retrieving
        location information
    """
    location = Location(connect_dict=get_connect_dict())
    info = location.retrieve_by_slug(location_slug)

    assert info, f"Location slug {location_slug} not found"
    assert "venue" in info, f"'venue' was not returned for slug {location_slug}"
    assert "coordinates" in info, (
        f"'coordinates' was not returned for slug {location_slug}"
    )
    if info["coordinates"]:
        assert "latitude" in info["coordinates"], (
            f"'latitude' was not returned for slug {location_slug}"
        )
        assert "longitude" in info["coordinates"], (
            f"'longitude' was not returned for slug {location_slug}"
        )


@pytest.mark.parametrize("location_slug", ["the-chicago-theatre-chicago-il"])
def test_location_retrieve_details_by_slug(location_slug: str):
    """Testing for :py:meth:`wwdtm.location.Location.retrieve_details_by_slug`.

    :param location_slug: Location slug string to test retrieving
        location details
    """
    location = Location(connect_dict=get_connect_dict())
    info = location.retrieve_details_by_slug(location_slug)

    assert info, f"Location slug {location_slug} not found"
    assert "venue" in info, f"'venue' was not returned for slug {location_slug}"
    assert "coordinates" in info, (
        f"'coordinates' was not returned for slug {location_slug}"
    )
    if info["coordinates"]:
        assert "latitude" in info["coordinates"], (
            f"'latitude' was not returned for slug {location_slug}"
        )
        assert "longitude" in info["coordinates"], (
            f"'longitude' was not returned for slug {location_slug}"
        )
    assert "recordings" in info, (
        f"'recordings' was not returned for slug {location_slug}"
    )


def test_location_retrieve_postal_abbreviations():
    """Testing for :py:meth:`wwdtm.location.Location.retrieve_postal_abbreviations`."""
    location = Location(connect_dict=get_connect_dict())
    abbreviations = location.retrieve_postal_abbreviations()

    assert abbreviations, "Postal abbreviations not returned"
    assert "OR" in abbreviations, "Postal abbreviation 'OR' not found"
    assert "name" in abbreviations["OR"], (
        "Postal abbreviation 'OR' does not contain a valid name"
    )


def test_location_retrieve_postal_abbreviations_list():
    """Testing for :py:meth:`wwdtm.location.Location.retrieve_postal_abbreviations_list`."""
    location = Location(connect_dict=get_connect_dict())
    abbreviations = location.retrieve_postal_abbreviations_list()

    assert abbreviations, "Postal abbreviations not returned"
    assert isinstance(abbreviations, list), "Returned data is not a list"
    assert "name" in abbreviations[0], "First item in the list does not include 'name'"


@pytest.mark.parametrize("abbreviation", ["OR", "DC"])
def test_location_retrieve_postal_details_by_abbreviation(abbreviation: str):
    """Testing for :py:meth:`wwdtm.location.Location.retrieve_postal_details_by_abbreviation`."""
    location = Location(connect_dict=get_connect_dict())
    details = location.retrieve_postal_details_by_abbreviation(
        abbreviation=abbreviation
    )

    assert details, f"Postal abbreviation {abbreviation} was not found"
    assert "postal_abbreviation" in details, (
        f"'postal_abbreviation' was not returned for {abbreviation}"
    )
    assert "name" in details, f"'name' was not returned for {abbreviation}"
    assert "country" in details, f"'country' was not returned for {abbreviation}"


def test_location_retrieve_random_id() -> None:
    """Testing for :py:meth`wwdtm.location.Location.retrieve_random_id`."""
    location = Location(connect_dict=get_connect_dict())
    _id = location.retrieve_random_id()

    assert _id, "Returned random location ID is not valid"
    assert isinstance(_id, int), "Returned random location ID is not an integer"


def test_location_retrieve_random_slug() -> None:
    """Testing for :py:meth`wwdtm.location.Location.retrieve_random_slug`."""
    location = Location(connect_dict=get_connect_dict())
    _slug = location.retrieve_random_slug()

    assert _slug, "Returned random location slug string is not valid"
    assert isinstance(_slug, str), (
        "Returned random location slug string is not a string"
    )


def test_location_retrieve_random() -> None:
    """Testing for :py:meth:`wwdtm.location.Location.retrieve_random`."""
    location = Location(connect_dict=get_connect_dict())
    info = location.retrieve_random()

    assert info, "Random location not found"
    assert "venue" in info, "'venue' attribute was not returned for a random location"


def test_location_retrieve_random_details() -> None:
    """Testing for :py:meth:`wwdtm.host.Location.retrieve_random_details`."""
    location = Location(connect_dict=get_connect_dict())
    info = location.retrieve_random_details()

    assert info, "Random location not found"
    assert "venue" in info, "'venue' attribute was not returned for a random location"
    assert "coordinates" in info, "'coordinates' was not returned for a random location"



# Copyright (c) 2018-2026 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
# SPDX-License-Identifier: Apache-2.0
#
# vim: set noai syntax=python ts=4 sw=4:
"""Explicitly listing all modules in this package."""

from typing import Any

from mysql.connector import connect
from mysql.connector.connection import MySQLConnection
from mysql.connector.pooling import PooledMySQLConnection

from wwdtm import validation
from wwdtm.guest import Guest, GuestAppearances, GuestUtility
from wwdtm.host import Host, HostAppearances, HostUtility
from wwdtm.location import Location, LocationRecordings, LocationUtility
from wwdtm.panelist import (
    Panelist,
    PanelistAppearances,
    PanelistDecimalScores,
    PanelistScores,
    PanelistStatistics,
    PanelistUtility,
)
from wwdtm.scorekeeper import Scorekeeper, ScorekeeperAppearances, ScorekeeperUtility
from wwdtm.show import Show, ShowInfo, ShowInfoMultiple, ShowUtility

VERSION = "2.22.0"


def database_version(
    connect_dict: dict[str, Any] = None,
    database_connection: MySQLConnection | PooledMySQLConnection = None,
) -> tuple[int] | None:
    """Returns Stats Database version, if available.

    :param connect_dict: A dictionary containing database connection
        settings as required by MySQL Connector/Python
    :param database_connection: MySQL database connection object
    :return: A tuple containg major, minor and revision numbers if
        available. If a version number is not available, None is
        returned.
    """
    if connect_dict:
        _connect_dict = connect_dict
        _database_connection = connect(**_connect_dict)
    elif database_connection:
        _database_connection = database_connection
        if not _database_connection.is_connected():
            _database_connection.reconnect()

    cursor = _database_connection.cursor(dictionary=True)
    query = """
            SELECT keyname, value
            FROM __metadata
            WHERE keyname = 'database_version'
            ORDER BY id DESC LIMIT 1;
            """
    cursor.execute(query)
    result = cursor.fetchone()

    if not result:
        return None

    version_info = str(result["value"]).split(".")
    if len(version_info) == 3:
        return int(version_info[0]), int(version_info[1]), int(version_info[2])
    elif len(version_info) == 2:
        return int(version_info[0]), int(version_info[1]), 0

    return None



# Copyright (c) 2018-2026 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
# SPDX-License-Identifier: Apache-2.0
#
# vim: set noai syntax=python ts=4 sw=4:
"""pytest conftest.py File."""

from wwdtm.guest import Guest
from wwdtm.host import Host
from wwdtm.location import Location
from wwdtm.panelist import Panelist, PanelistScores
from wwdtm.scorekeeper import Scorekeeper
from wwdtm.show import Show



# Copyright (c) 2018-2026 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
# SPDX-License-Identifier: Apache-2.0
#
# vim: set noai syntax=python ts=4 sw=4:
# pylint: disable=C0209
"""Wait Wait Stats Show Detailed Information Retrieval Functions for Multiple Shows."""

from typing import Any

from mysql.connector import connect
from mysql.connector.connection import MySQLConnection
from mysql.connector.pooling import PooledMySQLConnection
from slugify import slugify

from wwdtm.location.location import LocationUtility
from wwdtm.show.utility import ShowUtility
from wwdtm.validation import valid_int_id


class ShowInfoMultiple:
    """Multiple show information retrieval class.

    Contains methods used to retrieve panelist, guest and Bluff the
    Listener information for multiple shows.

    :param connect_dict: A dictionary containing database connection
        settings as required by MySQL Connector/Python
    :param database_connection: MySQL database connection object
    """

    def __init__(
        self,
        connect_dict: dict[str, Any] = None,
        database_connection: MySQLConnection | PooledMySQLConnection = None,
    ):
        """Class initialization method."""
        if connect_dict:
            self.connect_dict = connect_dict
            self.database_connection = connect(**connect_dict)
        elif database_connection:
            if not database_connection.is_connected():
                database_connection.reconnect()

            self.database_connection = database_connection

        self.utility = ShowUtility(database_connection=self.database_connection)
        self.loc_util = LocationUtility(database_connection=self.database_connection)
        self.panelists = self._retrieve_panelists()

    def _retrieve_panelists(self) -> dict[int, dict[str, str]]:
        """Retrieves panelist basic information.

        :return: A dictionary with panelist ID as the key and a
            dictionary with name and slug string as the value
        """
        query = """
            SELECT panelistid, panelist, panelistslug
            FROM ww_panelists
            ORDER BY panelistid ASC;
        """
        cursor = self.database_connection.cursor(dictionary=True)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return {}

        panelists = {}
        for row in results:
            panelists[row["panelistid"]] = {
                "name": row["panelist"],
                "slug": row["panelistslug"],
            }

        return panelists

    def retrieve_bluff_info_all(self) -> dict[int, list[dict[str, Any]]]:
        """Retrieves Bluff the Listener information for all shows.

        :return: A dictionary containing Bluff the Listener segment ID
            and information about the chosen Bluff panelist and correct
            Bluff panelist.
        """
        query = """
            SELECT blm.showid, blm.segment, blm.chosenbluffpnlid AS chosen_id,
            blm.correctbluffpnlid AS correct_id
            FROM ww_showbluffmap blm
            JOIN ww_shows s on s.showid = blm.showid
            ORDER BY s.showid ASC, blm.segment ASC;
            """
        cursor = self.database_connection.cursor(dictionary=True)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return {}

        bluff_info = {}
        for row in results:
            if row["showid"] not in bluff_info:
                bluff_info[row["showid"]] = []

            if not row["chosen_id"] and not row["correct_id"]:
                bluff_info[row["showid"]].append(
                    {
                        "segment": row["segment"],
                        "chosen_panelist": None,
                        "correct_panelist": None,
                    }
                )
            elif row["chosen_id"] and not row["correct_id"]:
                bluff_info[row["showid"]].append(
                    {
                        "segment": row["segment"],
                        "chosen_panelist": {
                            "id": row["chosen_id"],
                            "name": self.panelists[row["chosen_id"]]["name"],
                            "slug": (
                                self.panelists[row["chosen_id"]]["slug"]
                                if self.panelists[row["chosen_id"]]["slug"]
                                else slugify(self.panelists[row["chosen_id"]]["name"])
                            ),
                        },
                        "correct_panelist": None,
                    }
                )
            elif row["correct_id"] and not row["chosen_id"]:
                bluff_info[row["showid"]].append(
                    {
                        "segment": row["segment"],
                        "chosen_panelist": None,
                        "correct_panelist": {
                            "id": row["correct_id"],
                            "name": self.panelists[row["correct_id"]]["name"],
                            "slug": (
                                self.panelists[row["correct_id"]]["slug"]
                                if self.panelists[row["correct_id"]]["slug"]
                                else slugify(self.panelists[row["correct_id"]]["name"])
                            ),
                        },
                    }
                )
            else:
                bluff_info[row["showid"]].append(
                    {
                        "segment": row["segment"],
                        "chosen_panelist": {
                            "id": row["chosen_id"],
                            "name": self.panelists[row["chosen_id"]]["name"],
                            "slug": (
                                self.panelists[row["chosen_id"]]["slug"]
                                if self.panelists[row["chosen_id"]]["slug"]
                                else slugify(self.panelists[row["chosen_id"]]["name"])
                            ),
                        },
                        "correct_panelist": {
                            "id": row["correct_id"],
                            "name": self.panelists[row["correct_id"]]["name"],
                            "slug": (
                                self.panelists[row["correct_id"]]["slug"]
                                if self.panelists[row["correct_id"]]["slug"]
                                else slugify(self.panelists[row["correct_id"]]["name"])
                            ),
                        },
                    }
                )

        return bluff_info

    def retrieve_bluff_info_by_ids(
        self, show_ids: list[int]
    ) -> dict[int, list[dict[str, Any]]]:
        """Retrieves Bluff the Listener information for a list of shows.

        :param show_id: A list of show IDs
        :return: A dictionary containing Bluff the Listener segment ID
            and information about the chosen Bluff panelist and correct
            Bluff panelist.
        """
        for show_id in show_ids:
            if not valid_int_id(show_id):
                return {}

        query = """
            SELECT showid, segment, chosenbluffpnlid AS chosen_id,
            correctbluffpnlid AS correct_id
            FROM ww_showbluffmap
            WHERE showid IN ({ids})
            ORDER BY showid ASC, segment ASC;""".format(
            ids=", ".join(str(v) for v in show_ids)
        )
        cursor = self.database_connection.cursor(dictionary=True)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return {}

        bluff_info = {}
        for row in results:
            if row["showid"] not in bluff_info:
                bluff_info[row["showid"]] = []

            if not row["chosen_id"] and not row["correct_id"]:
                bluff_info[row["showid"]].append(
                    {
                        "segment": row["segment"],
                        "chosen_panelist": None,
                        "correct_panelist": None,
                    }
                )
            elif row["chosen_id"] and not row["correct_id"]:
                bluff_info[row["showid"]].append(
                    {
                        "segment": row["segment"],
                        "chosen_panelist": {
                            "id": row["chosen_id"],
                            "name": self.panelists[row["chosen_id"]]["name"],
                            "slug": (
                                self.panelists[row["chosen_id"]]["slug"]
                                if self.panelists[row["chosen_id"]]["slug"]
                                else slugify(self.panelists[row["chosen_id"]]["name"])
                            ),
                        },
                        "correct_panelist": None,
                    }
                )
            elif row["correct_id"] and not row["chosen_id"]:
                bluff_info[row["showid"]].append(
                    {
                        "segment": row["segment"],
                        "chosen_panelist": None,
                        "correct_panelist": {
                            "id": row["correct_id"],
                            "name": self.panelists[row["correct_id"]]["name"],
                            "slug": (
                                self.panelists[row["correct_id"]]["slug"]
                                if self.panelists[row["correct_id"]]["slug"]
                                else slugify(self.panelists[row["correct_id"]]["name"])
                            ),
                        },
                    }
                )
            else:
                bluff_info[row["showid"]].append(
                    {
                        "segment": row["segment"],
                        "chosen_panelist": {
                            "id": row["chosen_id"],
                            "name": self.panelists[row["chosen_id"]]["name"],
                            "slug": (
                                self.panelists[row["chosen_id"]]["slug"]
                                if self.panelists[row["chosen_id"]]["slug"]
                                else slugify(self.panelists[row["chosen_id"]]["name"])
                            ),
                        },
                        "correct_panelist": {
                            "id": row["correct_id"],
                            "name": self.panelists[row["correct_id"]]["name"],
                            "slug": (
                                self.panelists[row["correct_id"]]["slug"]
                                if self.panelists[row["correct_id"]]["slug"]
                                else slugify(self.panelists[row["correct_id"]]["name"])
                            ),
                        },
                    }
                )

        return bluff_info

    def retrieve_core_info_all(self) -> dict[int, dict[str, Any]]:
        """Retrieves core information for all shows.

        :return: A dictionary containing host, scorekeeper, location,
            description and notes
        """
        query = """
            SELECT s.showid AS show_id, s.showdate AS date,
            s.bestof AS best_of, s.repeatshowid AS repeat_show_id,
            s.showurl AS show_url,
            l.locationid AS location_id, l.city, l.state,
            pa.name AS state_name, l.venue, l.latitude, l.longitude,
            l.locationslug AS location_slug,
            h.hostid AS host_id, h.host, h.hostslug AS host_slug,
            hm.guest as host_guest,
            sk.scorekeeperid AS scorekeeper_id, sk.scorekeeper,
            sk.scorekeeperslug AS scorekeeper_slug,
            skm.guest AS scorekeeper_guest,
            skm.description AS scorekeeper_description,
            sd.showdescription AS show_description,
            sn.shownotes AS show_notes
            FROM ww_shows s
            JOIN ww_showlocationmap lm ON lm.showid = s.showid
            JOIN ww_locations l ON l.locationid = lm.locationid
            LEFT JOIN ww_postal_abbreviations pa ON pa.postal_abbreviation = l.state
            JOIN ww_showhostmap hm ON hm.showid = s.showid
            JOIN ww_hosts h ON h.hostid = hm.hostid
            JOIN ww_showskmap skm ON skm.showid = s.showid
            JOIN ww_scorekeepers sk ON sk.scorekeeperid = skm.scorekeeperid
            JOIN ww_showdescriptions sd ON sd.showid = s.showid
            JOIN ww_shownotes sn ON sn.showid = s.showid
            ORDER BY s.showdate ASC;
            """
        cursor = self.database_connection.cursor(dictionary=True)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return {}

        shows = {}
        for show in results:
            if not show["latitude"] and not show["longitude"]:
                coordinates = None
            else:
                coordinates = {
                    "latitude": show["latitude"] if show["latitude"] else None,
                    "longitude": show["longitude"] if show["longitude"] else None,
                }

            location_info = {
                "id": show["location_id"],
                "slug": show["location_slug"],
                "city": show["city"],
                "state": show["state"],
                "state_name": show["state_name"],
                "venue": show["venue"],
                "coordinates": coordinates,
            }

            if not show["location_slug"]:
                location_info["slug"] = self.loc_util.slugify_location(
                    location_id=show["location_id"],
                    venue=show["venue"],
                    city=show["city"],
                    state=show["state"],
                )

            host_info = {
                "id": show["host_id"],
                "name": show["host"],
                "slug": (
                    show["host_slug"] if show["host_slug"] else slugify(show["host"])
                ),
                "guest": bool(show["host_guest"]),
            }

            scorekeeper_info = {
                "id": show["scorekeeper_id"],
                "name": show["scorekeeper"],
                "slug": (
                    show["scorekeeper_slug"]
                    if show["scorekeeper_slug"]
                    else slugify(show["scorekeeper"])
                ),
                "guest": bool(show["scorekeeper_guest"]),
                "description": (
                    show["scorekeeper_description"]
                    if show["scorekeeper_description"]
                    else None
                ),
            }

            if show["show_description"]:
                description = str(show["show_description"]).strip()
            else:
                description = None

            notes = str(show["show_notes"]).strip() if show["show_notes"] else None

            show_info = {
                "id": show["show_id"],
                "date": show["date"].isoformat(),
                "best_of": bool(show["best_of"]),
                "repeat_show": bool(show["repeat_show_id"]),
                "original_show_id": None,
                "original_show_date": None,
                "show_url": show["show_url"],
                "description": description,
                "notes": notes,
                "location": location_info,
                "host": host_info,
                "scorekeeper": scorekeeper_info,
            }

            repeat_show_id = show["repeat_show_id"]
            if repeat_show_id:
                original_date = self.utility.convert_id_to_date(repeat_show_id)
                show_info["original_show_id"] = repeat_show_id
                show_info["original_show_date"] = original_date
            else:
                show_info.pop("original_show_id", None)
                show_info.pop("original_show_date", None)

            shows[show["show_id"]] = show_info

        return shows

    def retrieve_core_info_by_ids(
        self, show_ids: list[int]
    ) -> dict[int, dict[str, Any]]:
        """Retrieves core information for a list of shows.

        :param show_ids: A list of show IDs
        :return: A dictionary containing host, scorekeeper, location,
            description and notes
        """
        for show_id in show_ids:
            if not valid_int_id(show_id):
                return {}

        query = """
            SELECT s.showid AS show_id, s.showdate AS date,
            s.bestof AS best_of, s.repeatshowid AS repeat_show_id,
            s.showurl AS show_url,
            l.locationid AS location_id, l.city, l.state,
            pa.name AS state_name, l.venue, l.latitude, l.longitude,
            l.locationslug AS location_slug,
            h.hostid AS host_id, h.host, h.hostslug AS host_slug,
            hm.guest as host_guest,
            sk.scorekeeperid AS scorekeeper_id, sk.scorekeeper,
            sk.scorekeeperslug AS scorekeeper_slug,
            skm.guest AS scorekeeper_guest,
            skm.description AS scorekeeper_description,
            sd.showdescription AS show_description,
            sn.shownotes AS show_notes
            FROM ww_shows s
            JOIN ww_showlocationmap lm ON lm.showid = s.showid
            JOIN ww_locations l ON l.locationid = lm.locationid
            LEFT JOIN ww_postal_abbreviations pa ON pa.postal_abbreviation = l.state
            JOIN ww_showhostmap hm ON hm.showid = s.showid
            JOIN ww_hosts h ON h.hostid = hm.hostid
            JOIN ww_showskmap skm ON skm.showid = s.showid
            JOIN ww_scorekeepers sk ON sk.scorekeeperid = skm.scorekeeperid
            JOIN ww_showdescriptions sd ON sd.showid = s.showid
            JOIN ww_shownotes sn ON sn.showid = s.showid
            WHERE s.showid IN ({ids})
            ORDER BY s.showdate ASC;""".format(ids=", ".join(str(v) for v in show_ids))
        cursor = self.database_connection.cursor(dictionary=True)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return {}

        shows = {}
        for show in results:
            if not show["latitude"] and not show["longitude"]:
                coordinates = None
            else:
                coordinates = {
                    "latitude": show["latitude"] if show["latitude"] else None,
                    "longitude": show["longitude"] if show["longitude"] else None,
                }

            location_info = {
                "id": show["location_id"],
                "slug": show["location_slug"],
                "city": show["city"],
                "state": show["state"],
                "state_name": show["state_name"],
                "venue": show["venue"],
                "coordinates": coordinates,
            }

            if not show["location_slug"]:
                location_info["slug"] = self.loc_util.slugify_location(
                    location_id=show["location_id"],
                    venue=show["venue"],
                    city=show["city"],
                    state=show["state"],
                )

            host_info = {
                "id": show["host_id"],
                "name": show["host"],
                "slug": (
                    show["host_slug"] if show["host_slug"] else slugify(show["host"])
                ),
                "guest": bool(show["host_guest"]),
            }

            scorekeeper_info = {
                "id": show["scorekeeper_id"],
                "name": show["scorekeeper"],
                "slug": (
                    show["scorekeeper_slug"]
                    if show["scorekeeper_slug"]
                    else slugify(show["scorekeeper"])
                ),
                "guest": bool(show["scorekeeper_guest"]),
                "description": (
                    show["scorekeeper_description"]
                    if show["scorekeeper_description"]
                    else None
                ),
            }

            if show["show_description"]:
                description = str(show["show_description"]).strip()
            else:
                description = None

            notes = str(show["show_notes"]).strip() if show["show_notes"] else None

            show_info = {
                "id": show["show_id"],
                "date": show["date"].isoformat(),
                "best_of": bool(show["best_of"]),
                "repeat_show": bool(show["repeat_show_id"]),
                "original_show_id": None,
                "original_show_date": None,
                "show_url": show["show_url"],
                "description": description,
                "notes": notes,
                "location": location_info,
                "host": host_info,
                "scorekeeper": scorekeeper_info,
            }

            repeat_show_id = show["repeat_show_id"]
            if repeat_show_id:
                original_date = self.utility.convert_id_to_date(repeat_show_id)
                show_info["original_show_id"] = repeat_show_id
                show_info["original_show_date"] = original_date
            else:
                show_info.pop("original_show_id", None)
                show_info.pop("original_show_date", None)

            shows[show["show_id"]] = show_info

        return shows

    def retrieve_guest_info_all(self) -> dict[int, list[dict[str, Any]]]:
        """Retrieves Not My Job guest information for all shows.

        :return: A dictionary containing Not My Job guest information,
            including score and scoring exception for each guest
        """
        query = """
            SELECT s.showid AS show_id, gm.guestid AS guest_id,
            g.guest AS name, g.guestslug AS slug,
            gm.guestscore AS score, gm.exception AS score_exception
            FROM ww_showguestmap gm
            JOIN ww_guests g ON g.guestid = gm.guestid
            JOIN ww_shows s ON s.showid = gm.showid
            ORDER BY s.showdate ASC, gm.showguestmapid ASC;
            """
        cursor = self.database_connection.cursor(dictionary=True)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        shows = {}
        for guest in results:
            if guest["show_id"] not in shows:
                shows[guest["show_id"]] = []

            shows[guest["show_id"]].append(
                {
                    "id": guest["guest_id"],
                    "name": guest["name"],
                    "slug": guest["slug"] if guest["slug"] else slugify(guest["name"]),
                    "score": guest["score"],
                    "score_exception": bool(guest["score_exception"]),
                }
            )

        return shows

    def retrieve_guest_info_by_ids(
        self, show_ids: list[int]
    ) -> dict[int, list[dict[str, Any]]]:
        """Retrieves Not My Job guest information for all shows.

        :param show_ids: A list of show IDs
        :return: A dictionary containing Not My Job guest information,
            including score and scoring exception for each guest.
        """
        for show_id in show_ids:
            if not valid_int_id(show_id):
                return {}

        query = """
            SELECT s.showid AS show_id, gm.guestid AS guest_id,
            g.guest AS name, g.guestslug AS slug,
            gm.guestscore AS score, gm.exception AS score_exception
            FROM ww_showguestmap gm
            JOIN ww_guests g ON g.guestid = gm.guestid
            JOIN ww_shows s ON s.showid = gm.showid
            WHERE gm.showid IN ({ids})
            ORDER BY s.showdate ASC,
            gm.showguestmapid ASC;""".format(ids=", ".join(str(v) for v in show_ids))
        cursor = self.database_connection.cursor(dictionary=True)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        shows = {}
        for guest in results:
            if guest["show_id"] not in shows:
                shows[guest["show_id"]] = []

            shows[guest["show_id"]].append(
                {
                    "id": guest["guest_id"],
                    "name": guest["name"],
                    "slug": guest["slug"] if guest["slug"] else slugify(guest["name"]),
                    "score": guest["score"],
                    "score_exception": bool(guest["score_exception"]),
                }
            )

        return shows

    def retrieve_panelist_info_all(
        self, include_decimal_scores: bool = False
    ) -> dict[int, list[dict[str, Any]]]:
        """Retrieves panelist information for all shows.

        :param use_decimal_scores: A boolean to determine if decimal
            scores should be used and returned instead of integer scores
        :return: A dictionary containing panelist information, scores
            and rankings
        """
        if include_decimal_scores:
            query = """
                SELECT s.showid AS show_id, pm.panelistid AS panelist_id,
                p.panelist AS name, p.panelistslug AS slug,
                pm.panelistlrndstart AS start,
                pm.panelistlrndstart AS start_decimal,
                pm.panelistlrndcorrect AS correct,
                pm.panelistlrndcorrect AS correct_decimal,
                pm.panelistscore AS score,
                pm.panelistscore_decimal AS score_decimal,
                pm.showpnlrank AS pnl_rank
                FROM ww_showpnlmap pm
                JOIN ww_panelists p ON p.panelistid = pm.panelistid
                JOIN ww_shows s ON s.showid = pm.showid
                ORDER by s.showdate ASC, pm.panelistscore_decimal DESC,
                pm.showpnlmapid ASC;
                """
        else:
            query = """
                SELECT s.showid AS show_id, pm.panelistid AS panelist_id,
                p.panelist AS name, p.panelistslug AS slug,
                pm.panelistlrndstart AS start,
                pm.panelistlrndcorrect AS correct,
                pm.panelistscore AS score,
                pm.showpnlrank AS pnl_rank
                FROM ww_showpnlmap pm
                JOIN ww_panelists p ON p.panelistid = pm.panelistid
                JOIN ww_shows s ON s.showid = pm.showid
                ORDER by s.showdate ASC, pm.panelistscore DESC,
                pm.showpnlmapid ASC;
                """
        cursor = self.database_connection.cursor(dictionary=True)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return {}

        panelists = {}
        for panelist in results:
            if panelist["show_id"] not in panelists:
                panelists[panelist["show_id"]] = []

            _score_exception = False
            if include_decimal_scores:
                start = panelist.get("start_decimal")
                correct = panelist.get("correct_decimal")
                score = panelist.get("score_decimal")
                if start and correct and score and score != (start + (correct * 2)):
                    _score_exception = True
            else:
                start = panelist.get("start")
                correct = panelist.get("correct")
                score = panelist.get("score")
                if start and correct and score and score != (start + (correct * 2)):
                    _score_exception = True

            panelists[panelist["show_id"]].append(
                {
                    "id": panelist["panelist_id"],
                    "name": panelist["name"],
                    "slug": (
                        panelist["slug"]
                        if panelist["slug"]
                        else slugify(panelist["name"])
                    ),
                    "lightning_round_start": panelist["start"],
                    "lightning_round_start_decimal": panelist.get(
                        "start_decimal", None
                    ),
                    "lightning_round_correct": panelist["correct"],
                    "lightning_round_correct_decimal": panelist.get(
                        "correct_decimal", None
                    ),
                    "score": panelist["score"],
                    "score_decimal": panelist.get("score_decimal", None),
                    "score_exception": _score_exception,
                    "rank": panelist["pnl_rank"] if panelist["pnl_rank"] else None,
                }
            )

        return panelists

    def retrieve_panelist_info_by_ids(
        self, show_ids: list[int], include_decimal_scores: bool = False
    ) -> dict[int, list[dict[str, Any]]]:
        """Retrieves panelist information for a list of shows.

        :param show_ids: A list of show IDs
        :param use_decimal_scores: A boolean to determine if decimal
            scores should be used and returned instead of integer scores
        :return: A dictionary containing panelist information, scores
            and rankings
        """
        for show_id in show_ids:
            if not valid_int_id(show_id):
                return {}

        if include_decimal_scores:
            query = """
                SELECT s.showid AS show_id, pm.panelistid AS panelist_id,
                p.panelist AS name, p.panelistslug AS slug,
                pm.panelistlrndstart AS start,
                pm.panelistlrndstart AS start_decimal,
                pm.panelistlrndcorrect AS correct,
                pm.panelistlrndcorrect AS correct_decimal,
                pm.panelistscore AS score,
                pm.panelistscore_decimal AS score_decimal,
                pm.showpnlrank AS pnl_rank
                FROM ww_showpnlmap pm
                JOIN ww_panelists p ON p.panelistid = pm.panelistid
                JOIN ww_shows s ON s.showid = pm.showid
                WHERE pm.showid IN ({ids})
                ORDER by s.showdate ASC, pm.panelistscore_decimal DESC,
                pm.showpnlmapid ASC;""".format(ids=", ".join(str(v) for v in show_ids))
        else:
            query = """
                SELECT s.showid AS show_id, pm.panelistid AS panelist_id,
                p.panelist AS name, p.panelistslug AS slug,
                pm.panelistlrndstart AS start,
                pm.panelistlrndcorrect AS correct,
                pm.panelistscore AS score, pm.showpnlrank AS pnl_rank
                FROM ww_showpnlmap pm
                JOIN ww_panelists p ON p.panelistid = pm.panelistid
                JOIN ww_shows s ON s.showid = pm.showid
                WHERE pm.showid IN ({ids})
                ORDER by s.showdate ASC, pm.panelistscore DESC,
                pm.showpnlmapid ASC;""".format(ids=", ".join(str(v) for v in show_ids))

        cursor = self.database_connection.cursor(dictionary=True)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return {}

        panelists = {}
        for panelist in results:
            if panelist["show_id"] not in panelists:
                panelists[panelist["show_id"]] = []

            _score_exception = False
            if include_decimal_scores:
                start = panelist.get("start_decimal")
                correct = panelist.get("correct_decimal")
                score = panelist.get("score_decimal")
                if start and correct and score and score != (start + (correct * 2)):
                    _score_exception = True
            else:
                start = panelist.get("start")
                correct = panelist.get("correct")
                score = panelist.get("score")
                if start and correct and score and score != (start + (correct * 2)):
                    _score_exception = True

            panelists[panelist["show_id"]].append(
                {
                    "id": panelist["panelist_id"],
                    "name": panelist["name"],
                    "slug": (
                        panelist["slug"]
                        if panelist["slug"]
                        else slugify(panelist["name"])
                    ),
                    "lightning_round_start": (
                        panelist["start"] if panelist["start"] else None
                    ),
                    "lightning_round_start_decimal": panelist.get(
                        "start_decimal", None
                    ),
                    "lightning_round_correct": (
                        panelist["correct"] if panelist["correct"] else None
                    ),
                    "lightning_round_correct_decimal": panelist.get(
                        "correct_decimal", None
                    ),
                    "score": panelist["score"],
                    "score_decimal": panelist.get("score_decimal", None),
                    "score_exception": _score_exception,
                    "rank": panelist["pnl_rank"] if panelist["pnl_rank"] else None,
                }
            )

        return panelists



# Copyright (c) 2018-2026 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
# SPDX-License-Identifier: Apache-2.0
#
# vim: set noai syntax=python ts=4 sw=4:
"""Testing for object: :py:class:`wwdtm.panelist.PanelistAppearances`."""

import json
from pathlib import Path
from typing import Any

import pytest

from wwdtm.panelist import PanelistAppearances


@pytest.mark.skip
def get_connect_dict() -> dict[str, Any]:
    """Retrieves database connection settings.

    :return: A dictionary containing database connection
        settings as required by MySQL Connector/Python
    """
    file_path = Path.cwd() / "config.json"
    with file_path.open(mode="r", encoding="utf-8") as config_file:
        config_dict = json.load(config_file)
        if "database" in config_dict:
            return config_dict["database"]


@pytest.mark.parametrize(
    "panelist_id, use_decimal_scores",
    [(14, True), (14, False), (73, True), (73, False)],
)
def test_panelist_appearances_retrieve_appearances_by_id(
    panelist_id: int, use_decimal_scores: bool
):
    """Testing for :py:meth:`wwdtm.panelist.PanelistAppearances.retrieve_appearances_by_id`.

    :param panelist_id: Panelist ID to test retrieving panelist
        appearances
    :param use_decimal_scores: Flag set to use decimal score columns
        and values
    """
    appearances = PanelistAppearances(connect_dict=get_connect_dict())
    appearance = appearances.retrieve_appearances_by_id(
        panelist_id, use_decimal_scores=use_decimal_scores
    )

    assert "count" in appearance, f"'count' was not returned for ID {panelist_id}"
    assert "shows" in appearance, f"'shows' was not returned for ID {panelist_id}"


@pytest.mark.parametrize(
    "panelist_slug, use_decimal_scores",
    [
        ("luke-burbank", True),
        ("luke-burbank", False),
        ("maeve-higgins", True),
        ("maeve-higgins", False),
        ("peter-sagal", True),
        ("peter-sagal", False),
    ],
)
def test_panelist_appearances_retrieve_appearances_by_slug(
    panelist_slug: str, use_decimal_scores: bool
):
    """Testing for :py:meth:`wwdtm.panelist.PanelistAppearances.retrieve_appearances_by_slug`.

    :param panelist_slug: Panelist slug string to test retrieving
        panelist appearances
    :param use_decimal_scores: Flag set to use decimal score columns
        and values
    """
    appearances = PanelistAppearances(connect_dict=get_connect_dict())
    appearance = appearances.retrieve_appearances_by_slug(
        panelist_slug, use_decimal_scores=use_decimal_scores
    )

    assert "count" in appearance, f"'count' was not returned for slug {panelist_slug}"
    assert "shows" in appearance, f"'shows' was not returned for slug {panelist_slug}"


@pytest.mark.parametrize("panelist_id", [14, 73])
def test_panelist_appearances_retrieve_yearly_appearances_by_id(panelist_id: int):
    """Testing for :py:meth:`wwdtm.panelist.PanelistAppearances.retrieve_yearly_appearances_by_id`.

    :param panelist_id: Panelist ID to test retrieving a panelist's
        appearances
    :param use_decimal_scores: Flag set to use decimal score columns
        and values
    """
    appearances = PanelistAppearances(connect_dict=get_connect_dict())
    breakdown = appearances.retrieve_yearly_appearances_by_id(panelist_id)

    assert breakdown, f"No appearance information returned for ID {panelist_id}"


@pytest.mark.parametrize("panelist_slug", ["luke-burbank", "maeve-higgins"])
def test_panelist_appearances_retrieve_yearly_appearances_by_slug(panelist_slug: str):
    """Testing for :py:meth:`wwdtm.panelist.PanelistAppearances.retrieve_yearly_appearances_by_slug`.

    :param panelist_slug: Panelist slug string to test retrieving
        panelist appearances
    """
    appearances = PanelistAppearances(connect_dict=get_connect_dict())
    breakdown = appearances.retrieve_yearly_appearances_by_slug(panelist_slug)

    assert breakdown, f"No appearance information returned for slug {panelist_slug}"



# Copyright (c) 2018-2026 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
# SPDX-License-Identifier: Apache-2.0
#
# vim: set noai syntax=python ts=4 sw=4:
"""Testing for object: :py:class:`wwdtm.panelist.PanelistStatistics`."""

import json
from pathlib import Path
from typing import Any

import pytest

from wwdtm.panelist import PanelistStatistics


@pytest.mark.skip
def get_connect_dict() -> dict[str, Any]:
    """Retrieves database connection settings.

    :return: A dictionary containing database connection
        settings as required by MySQL Connector/Python
    """
    file_path = Path.cwd() / "config.json"
    with file_path.open(mode="r", encoding="utf-8") as config_file:
        config_dict = json.load(config_file)
        if "database" in config_dict:
            return config_dict["database"]


@pytest.mark.parametrize("panelist_id", [14])
def test_panelist_statistics_retrieve_bluffs_by_id(panelist_id: int):
    """Testing for :py:meth:`wwdtm.panelist.PanelistStatistics.retrieve_bluffs_by_id`.

    :param panelist_id: Panelist ID to test retrieving panelist
        information
    """
    statistics = PanelistStatistics(connect_dict=get_connect_dict())
    bluffs = statistics.retrieve_bluffs_by_id(panelist_id)

    assert "chosen" in bluffs, f"'chosen' was not returned for ID {panelist_id}"
    assert "correct" in bluffs, f"'correct' was not returned for ID {panelist_id}"


@pytest.mark.parametrize("panelist_slug", ["luke-burbank"])
def test_panelist_statistics_retrieve_bluffs_by_slug(panelist_slug: str):
    """Testing for :py:meth:`wwdtm.panelist.PanelistStatistics.retrieve_bluffs_by_slug`.

    :param panelist_slug: Panelist slug string to test retrieving
        panelist information
    """
    statistics = PanelistStatistics(connect_dict=get_connect_dict())
    bluffs = statistics.retrieve_bluffs_by_slug(panelist_slug)

    assert "chosen" in bluffs, f"'chosen' was not returned for slug {panelist_slug}"
    assert "correct" in bluffs, f"'correct' was not returned for slug {panelist_slug}"


@pytest.mark.parametrize("panelist_id", [14])
def test_panelist_statistics_retrieve_rank_info_by_id(panelist_id: int):
    """Testing for :py:meth:`wwdtm.panelist.PanelistStatistics.retrieve_rank_info_by_id`.

    :param panelist_id: Panelist ID to test retrieving panelist
        information
    """
    statistics = PanelistStatistics(connect_dict=get_connect_dict())
    ranks = statistics.retrieve_rank_info_by_id(panelist_id)

    assert "first" in ranks, f"'first' was not returned for ID {panelist_id}"


@pytest.mark.parametrize("panelist_slug", ["luke-burbank"])
def test_panelist_statistics_retrieve_rank_info_by_slug(panelist_slug: str):
    """Testing for :py:meth:`wwdtm.panelist.PanelistStatistics.retrieve_rank_info_by_slug`.

    :param panelist_slug: Panelist slug string to test retrieving
        panelist information
    """
    statistics = PanelistStatistics(connect_dict=get_connect_dict())
    ranks = statistics.retrieve_rank_info_by_slug(panelist_slug)

    assert "first" in ranks, f"'first' was not returned for slug {panelist_slug}"


@pytest.mark.parametrize(
    "panelist_id, include_decimal_scores", [(14, True), (14, False)]
)
def test_panelist_statistics_retrieve_statistics_by_id(
    panelist_id: int, include_decimal_scores: bool
):
    """Testing for :py:meth:`wwdtm.panelist.PanelistStatistics.retrieve_statistics_by_id`.

    :param panelist_id: Panelist ID to test retrieving panelist
        information
    :param include_decimal_scores: Flag set to include decimal score columns
        and values
    """
    statistics = PanelistStatistics(connect_dict=get_connect_dict())
    stats = statistics.retrieve_statistics_by_id(
        panelist_id, include_decimal_scores=include_decimal_scores
    )

    assert "scoring" in stats, f"'scoring' was not returned for ID {panelist_id}"
    if include_decimal_scores:
        assert "scoring_decimal" in stats, (
            f"'scoring' was not returned for ID {panelist_id}"
        )
    assert "ranking" in stats, f"'ranking' was not returned for ID {panelist_id}"


@pytest.mark.parametrize(
    "panelist_slug, include_decimal_scores",
    [("luke-burbank", True), ("luke-burbank", False)],
)
def test_panelist_statistics_retrieve_statistics_by_slug(
    panelist_slug: str, include_decimal_scores: bool
):
    """Testing for :py:meth:`wwdtm.panelist.PanelistStatistics.retrieve_statistics_by_slug`.

    :param panelist_slug: Panelist slug string to test retrieving
        panelist information
    :param include_decimal_scores: Flag set to include decimal score columns
        and values
    """
    statistics = PanelistStatistics(connect_dict=get_connect_dict())
    stats = statistics.retrieve_statistics_by_slug(
        panelist_slug, include_decimal_scores=include_decimal_scores
    )

    assert "scoring" in stats, f"'scoring' was not returned for slug {panelist_slug}"
    if include_decimal_scores:
        assert "scoring_decimal" in stats, (
            f"'scoring' was not returned for ID {panelist_slug}"
        )
    assert "ranking" in stats, f"'ranking' was not returned for slug {panelist_slug}"



# Copyright (c) 2018-2026 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
# SPDX-License-Identifier: Apache-2.0
#
# vim: set noai syntax=python ts=4 sw=4:
"""Wait Wait Stats Host Data Retrieval Functions."""

from typing import Any

from mysql.connector import connect
from mysql.connector.connection import MySQLConnection
from mysql.connector.pooling import PooledMySQLConnection
from slugify import slugify

from wwdtm.guest.utility import GuestUtility
from wwdtm.host.appearances import HostAppearances
from wwdtm.host.utility import HostUtility
from wwdtm.panelist.utility import PanelistUtility
from wwdtm.scorekeeper.utility import ScorekeeperUtility
from wwdtm.validation import valid_int_id


class Host:
    """Host information retrieval class.

    Contains methods used to retrieve host information, including IDs,
    names, slug strings and appearances.

    :param connect_dict: A dictionary containing database connection
        settings as required by MySQL Connector/Python
    :param database_connection: MySQL database connection object
    """

    def __init__(
        self,
        connect_dict: dict[str, Any] = None,
        database_connection: MySQLConnection | PooledMySQLConnection = None,
    ):
        """Class initialization method."""
        if connect_dict:
            self.connect_dict = connect_dict
            self.database_connection = connect(**connect_dict)
        elif database_connection:
            if not database_connection.is_connected():
                database_connection.reconnect()

            self.database_connection = database_connection

        self.appearances = HostAppearances(database_connection=self.database_connection)
        self.utility = HostUtility(database_connection=self.database_connection)

    def retrieve_all(self) -> list[dict[str, int | str]]:
        """Retrieves host information for all hosts.

        :return: A list of dictionaries containing host ID, name, slug
            string, gender and pronouns for each host
        """
        query = """
            SELECT h.hostid AS id, h.host AS name, h.hostslug AS slug,
            hostgender AS gender
            FROM ww_hosts h
            ORDER BY host ASC;
            """
        cursor = self.database_connection.cursor(dictionary=True)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        hosts = []

        cursor = self.database_connection.cursor(dictionary=True)
        for row in results:
            query = """
                SELECT pn.pronouns
                FROM ww_hostpronounsmap hpm
                JOIN ww_pronouns pn on pn.pronounsid = hpm.pronounsid
                WHERE hpm.hostid = %s
                ORDER BY hpm.hostpronounsmapid ASC;
                """
            cursor.execute(query, (row["id"],))
            pn_results = cursor.fetchall()

            hosts.append(
                {
                    "id": row["id"],
                    "name": row["name"],
                    "slug": row["slug"] if row["slug"] else slugify(row["name"]),
                    "gender": row["gender"],
                    "pronouns": (
                        [result["pronouns"] for result in pn_results]
                        if pn_results
                        else []
                    ),
                }
            )

        cursor.close()

        return hosts

    def retrieve_all_details(self) -> list[dict[str, Any]]:
        """Retrieves host information and appearances for all hosts.

        :return: A list of dictionaries containing host ID, name, slug
            string, gender, pronouns, whether the host is also a guest,
            host or scorekeeper, and a list of appearances with show
            flags for each host
        """
        query = """
            SELECT h.hostid AS id, h.host AS name, h.hostslug AS slug,
            h.hostgender AS gender
            FROM ww_hosts h
            ORDER BY host ASC;
            """
        cursor = self.database_connection.cursor(dictionary=True)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        hosts = []
        _guest_utility = GuestUtility(database_connection=self.database_connection)
        _panelist_utility = PanelistUtility(
            database_connection=self.database_connection
        )
        _scorekeeper_utility = ScorekeeperUtility(
            database_connection=self.database_connection
        )

        cursor = self.database_connection.cursor(dictionary=True)
        for row in results:
            _slug = row["slug"] if row["slug"] else slugify(row["name"])
            query = """
                SELECT pn.pronouns
                FROM ww_hostpronounsmap hpm
                JOIN ww_pronouns pn on pn.pronounsid = hpm.pronounsid
                WHERE hpm.hostid = %s
                ORDER BY hpm.hostpronounsmapid ASC;
                """
            cursor.execute(query, (row["id"],))
            pn_results = cursor.fetchall()

            hosts.append(
                {
                    "id": row["id"],
                    "name": row["name"],
                    "slug": _slug,
                    "gender": row["gender"],
                    "pronouns": (
                        [result["pronouns"] for result in pn_results]
                        if pn_results
                        else []
                    ),
                    "is_guest": bool(_guest_utility.slug_exists(guest_slug=_slug)),
                    "is_panelist": bool(
                        _panelist_utility.slug_exists(panelist_slug=_slug)
                    ),
                    "is_scorekeeper": bool(
                        _scorekeeper_utility.slug_exists(scorekeeper_slug=_slug)
                    ),
                    "appearances": self.appearances.retrieve_appearances_by_id(
                        row["id"]
                    ),
                }
            )

        cursor.close()

        return hosts

    def retrieve_all_ids(self) -> list[int]:
        """Retrieves all host IDs, sorted by host name.

        :return: A list of host IDs as integers
        """
        cursor = self.database_connection.cursor(dictionary=False)
        query = "SELECT hostid FROM ww_hosts ORDER BY host ASC;"
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        return [v[0] for v in results]

    def retrieve_all_slugs(self) -> list[str]:
        """Retrieves all host slug strings, sorted by host name.

        :return: A list of host slug strings
        """
        query = """
            SELECT hostslug FROM ww_hosts ORDER BY host ASC;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        return [v[0] for v in results]

    def retrieve_by_id(self, host_id: int) -> dict[str, Any]:
        """Retrieves host information.

        :param host_id: Host ID
        :return: A dictionary containing host ID, name, gender and slug string
        """
        if not valid_int_id(host_id):
            return {}

        query = """
            SELECT h.hostid AS id, h.host AS name, h.hostslug AS slug,
            h.hostgender AS gender
            FROM ww_hosts h
            WHERE h.hostid = %s
            LIMIT 1;
            """
        cursor = self.database_connection.cursor(dictionary=True)
        cursor.execute(query, (host_id,))
        result = cursor.fetchone()

        if not result:
            return {}

        query = """
            SELECT pn.pronouns
            FROM ww_hostpronounsmap hpm
            JOIN ww_pronouns pn on pn.pronounsid = hpm.pronounsid
            WHERE hpm.hostid = %s
            ORDER BY hpm.hostpronounsmapid ASC;
            """
        cursor = self.database_connection.cursor(dictionary=True)
        cursor.execute(query, (host_id,))
        results = cursor.fetchall()
        cursor.close()

        return {
            "id": result["id"],
            "name": result["name"],
            "slug": result["slug"] if result["slug"] else slugify(result["name"]),
            "gender": result["gender"],
            "pronouns": [result["pronouns"] for result in results] if results else [],
        }

    def retrieve_by_slug(self, host_slug: str) -> dict[str, Any]:
        """Retrieves host information.

        :param host_slug: Host slug string
        :return: A dictionary containing host ID, name, gender and slug
            string
        """
        try:
            slug = host_slug.strip()
            if not slug:
                return {}
        except AttributeError:
            return {}

        id_ = self.utility.convert_slug_to_id(slug)
        if not id_:
            return {}

        return self.retrieve_by_id(id_)

    def retrieve_details_by_id(self, host_id: int) -> dict[str, Any]:
        """Retrieves host information and appearances.

        :param host_id: Host ID
        :return: A dictionary containing host ID, name, slug string,
            gender, pronouns, whether the host is also a guest, host or
            scorekeeper, and a list of appearances with show flags
        """
        if not valid_int_id(host_id):
            return {}

        info = self.retrieve_by_id(host_id)
        if not info:
            return {}

        _guest_utility = GuestUtility(database_connection=self.database_connection)
        _panelist_utility = PanelistUtility(
            database_connection=self.database_connection
        )
        _scorekeeper_utility = ScorekeeperUtility(
            database_connection=self.database_connection
        )

        info["is_guest"] = bool(_guest_utility.slug_exists(guest_slug=info["slug"]))
        info["is_panelist"] = bool(
            _panelist_utility.slug_exists(panelist_slug=info["slug"])
        )
        info["is_scorekeeper"] = bool(
            _scorekeeper_utility.slug_exists(scorekeeper_slug=info["slug"])
        )
        info["appearances"] = self.appearances.retrieve_appearances_by_id(host_id)

        return info

    def retrieve_details_by_slug(self, host_slug: str) -> dict[str, Any]:
        """Retrieves host information and appearances.

        :param host_slug: Host slug string
        :return: A dictionary containing host ID, name, slug string,
            gender, pronouns, whether the host is also a guest, host or
            scorekeeper, and a list of appearances with show flags
        """
        try:
            slug = host_slug.strip()
            if not slug:
                return {}
        except AttributeError:
            return {}

        id_ = self.utility.convert_slug_to_id(slug)
        if not id_:
            return {}

        return self.retrieve_details_by_id(id_)

    def retrieve_random_id(self) -> int:
        """Retrieves an ID for a random host.

        :return: ID for a random host.
        """
        query = """
            SELECT hostid FROM ww_hosts
            WHERE hostslug <> 'tbd'
            ORDER BY RAND()
            LIMIT 1;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query)
        result = cursor.fetchone()
        cursor.close()

        if not result:
            return None

        return result[0]

    def retrieve_random_slug(self) -> str:
        """Retrieves an slug string for a random host.

        :return: Slug string for a random host.
        """
        query = """
            SELECT hostslug FROM ww_hosts
            WHERE hostslug <> 'tbd'
            ORDER BY RAND()
            LIMIT 1;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query)
        result = cursor.fetchone()
        cursor.close()

        if not result:
            return None

        return result[0]

    def retrieve_random(self) -> dict[str, Any]:
        """Retrieves information for a random host.

        :return: A dictionary containing host ID, name, gender and slug
            string
        """
        _id = self.retrieve_random_id()

        if not _id:
            return None

        return self.retrieve_by_id(host_id=_id)

    def retrieve_random_details(self) -> dict[str, Any]:
        """Retrieves information and appearances for a random host.

        :return: A dictionary containing host ID, name, slug string,
            gender, pronouns, whether the host is also a guest, host or
            scorekeeper, and a list of appearances with show flags
        """
        _id = self.retrieve_random_id()

        if not _id:
            return None

        return self.retrieve_details_by_id(host_id=_id)



# Copyright (c) 2018-2026 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
# SPDX-License-Identifier: Apache-2.0
#
# vim: set noai syntax=python ts=4 sw=4:
"""Wait Wait Stats Scorekeeper Data Retrieval Functions."""

from typing import Any

from mysql.connector import connect
from mysql.connector.connection import MySQLConnection
from mysql.connector.pooling import PooledMySQLConnection
from slugify import slugify

from wwdtm.guest.utility import GuestUtility
from wwdtm.host.utility import HostUtility
from wwdtm.panelist.utility import PanelistUtility
from wwdtm.scorekeeper.appearances import ScorekeeperAppearances
from wwdtm.scorekeeper.utility import ScorekeeperUtility
from wwdtm.validation import valid_int_id


class Scorekeeper:
    """Scorekeeper information retrieval class.

    Contains methods used to retrieve scorekeeper information, including
    IDs, names, slug strings and appearances.

    :param connect_dict: A dictionary containing database connection
        settings as required by MySQL Connector/Python
    :param database_connection: MySQL database connection object
    """

    def __init__(
        self,
        connect_dict: dict[str, Any] = None,
        database_connection: MySQLConnection | PooledMySQLConnection = None,
    ):
        """Class initialization method."""
        if connect_dict:
            self.connect_dict = connect_dict
            self.database_connection = connect(**connect_dict)
        elif database_connection:
            if not database_connection.is_connected():
                database_connection.reconnect()

            self.database_connection = database_connection

        self.appearances = ScorekeeperAppearances(
            database_connection=self.database_connection
        )
        self.utility = ScorekeeperUtility(database_connection=self.database_connection)

    def retrieve_all(self) -> list[dict[str, Any]]:
        """Retrieves scorekeeper information for all scorekeepers.

        :return: A list of dictionaries containing scorekeeper ID, name,
            gender, pronouns and slug string for each scorekeeper
        """
        query = """
            SELECT sk.scorekeeperid AS id, sk.scorekeeper AS name,
            sk.scorekeeperslug AS slug, sk.scorekeepergender AS gender
            FROM ww_scorekeepers sk
            ORDER BY scorekeeper ASC;
            """
        cursor = self.database_connection.cursor(dictionary=True)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        scorekeepers = []

        cursor = self.database_connection.cursor(dictionary=True)
        for row in results:
            query = """
                SELECT pn.pronouns
                FROM ww_skpronounsmap spm
                JOIN ww_pronouns pn on pn.pronounsid = spm.pronounsid
                WHERE spm.scorekeeperid = %s
                ORDER BY spm.skpronounsmapid ASC;
                """
            cursor.execute(query, (row["id"],))
            pn_results = cursor.fetchall()

            scorekeepers.append(
                {
                    "id": row["id"],
                    "name": row["name"],
                    "slug": row["slug"] if row["slug"] else slugify(row["name"]),
                    "gender": row["gender"],
                    "pronouns": (
                        [result["pronouns"] for result in pn_results]
                        if pn_results
                        else []
                    ),
                }
            )

        return scorekeepers

    def retrieve_all_details(self) -> list[dict[str, Any]]:
        """Retrieves scorekeeper information and appearances for all scorekeepers.

        :return: A list of dictionaries containing scorekeeper ID, name,
            slug string, gender, pronouns, whether the scorekeeper is also
            a guest, host or panelist, and a list of appearances with show
            flags for each scorekeeper
        """
        query = """
            SELECT sk.scorekeeperid AS id, sk.scorekeeper AS name,
            sk.scorekeeperslug AS slug, sk.scorekeepergender AS gender
            FROM ww_scorekeepers sk
            ORDER BY scorekeeper ASC;
            """
        cursor = self.database_connection.cursor(dictionary=True)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        scorekeepers = []
        _guest_utility = GuestUtility(database_connection=self.database_connection)
        _host_utility = HostUtility(database_connection=self.database_connection)
        _panelist_utility = PanelistUtility(
            database_connection=self.database_connection
        )

        cursor = self.database_connection.cursor(dictionary=True)
        for row in results:
            _slug = row["slug"] if row["slug"] else slugify(row["name"])
            query = """
                SELECT pn.pronouns
                FROM ww_skpronounsmap spm
                JOIN ww_pronouns pn on pn.pronounsid = spm.pronounsid
                WHERE spm.scorekeeperid = %s
                ORDER BY spm.skpronounsmapid ASC;
                """
            cursor.execute(query, (row["id"],))
            pn_results = cursor.fetchall()

            scorekeepers.append(
                {
                    "id": row["id"],
                    "name": row["name"],
                    "slug": _slug,
                    "gender": row["gender"],
                    "pronouns": (
                        [result["pronouns"] for result in pn_results]
                        if pn_results
                        else []
                    ),
                    "is_guest": bool(_guest_utility.slug_exists(guest_slug=_slug)),
                    "is_host": bool(_host_utility.slug_exists(host_slug=_slug)),
                    "is_panelist": bool(
                        _panelist_utility.slug_exists(panelist_slug=_slug)
                    ),
                    "appearances": self.appearances.retrieve_appearances_by_id(
                        row["id"]
                    ),
                }
            )

        return scorekeepers

    def retrieve_all_ids(self) -> list[int]:
        """Retrieves all scorekeeper IDs, sorted by scorekeeper name.

        :return: A list of scorekeeper IDs as integers
        """
        query = """
            SELECT scorekeeperid FROM ww_scorekeepers ORDER BY scorekeeper ASC;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        return [v[0] for v in results]

    def retrieve_all_slugs(self) -> list[str]:
        """Retrieves all scorekeeper slug strings, sorted by scorekeeper name.

        :return: A list of scorekeeper slug strings
        """
        query = """
            SELECT scorekeeperslug FROM ww_scorekeepers ORDER BY scorekeeper ASC;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query)
        results = cursor.fetchall()
        cursor.close()

        if not results:
            return []

        return [v[0] for v in results]

    def retrieve_by_id(self, scorekeeper_id: int) -> dict[str, Any]:
        """Retrieves scorekeeper information.

        :param scorekeeper_id: Scorekeeper ID
        :return: A dictionary containing scorekeeper ID, name, slug
            string, gender and pronouns
        """
        if not valid_int_id(scorekeeper_id):
            return {}

        query = """
            SELECT sk.scorekeeperid AS id, sk.scorekeeper AS name,
            sk.scorekeeperslug AS slug, sk.scorekeepergender AS gender
            FROM ww_scorekeepers sk
            WHERE sk.scorekeeperid = %s
            LIMIT 1;
            """
        cursor = self.database_connection.cursor(dictionary=True)
        cursor.execute(query, (scorekeeper_id,))
        result = cursor.fetchone()
        cursor.close()

        if not result:
            return {}

        query = """
            SELECT pn.pronouns
            FROM ww_skpronounsmap spm
            JOIN ww_pronouns pn on pn.pronounsid = spm.pronounsid
            WHERE spm.scorekeeperid = %s
            ORDER BY spm.skpronounsmapid ASC;
            """
        cursor = self.database_connection.cursor(dictionary=True)
        cursor.execute(query, (scorekeeper_id,))
        results = cursor.fetchall()
        cursor.close()

        return {
            "id": result["id"],
            "name": result["name"],
            "slug": result["slug"] if result["slug"] else slugify(result["name"]),
            "gender": result["gender"],
            "pronouns": [result["pronouns"] for result in results] if results else [],
        }

    def retrieve_by_slug(self, scorekeeper_slug: str) -> dict[str, Any]:
        """Retrieves scorekeeper information.

        :param scorekeeper_slug: Scorekeeper slug string
        :return: A dictionary containing scorekeeper ID, name, slug
            string, gender and pronouns
        """
        try:
            slug = scorekeeper_slug.strip()
            if not slug:
                return {}
        except AttributeError:
            return {}

        id_ = self.utility.convert_slug_to_id(slug)
        if not id_:
            return {}

        return self.retrieve_by_id(id_)

    def retrieve_details_by_id(self, scorekeeper_id: int) -> dict[str, Any]:
        """Retrieves scorekeeper information and appearances.

        :param scorekeeper_id: Scorekeeper ID
        :return: A dictionaries containing scorekeeper ID, name, slug
            string, gender, pronouns, whether the scorekeeper is also
            a guest, host or panelist, and a list of appearances with
            show flags
        """
        if not valid_int_id(scorekeeper_id):
            return {}

        info = self.retrieve_by_id(scorekeeper_id)
        if not info:
            return {}

        _guest_utility = GuestUtility(database_connection=self.database_connection)
        _host_utility = HostUtility(database_connection=self.database_connection)
        _panelist_utility = PanelistUtility(
            database_connection=self.database_connection
        )

        info["is_guest"] = bool(_guest_utility.slug_exists(info["slug"]))
        info["is_host"] = bool(_host_utility.slug_exists(host_slug=info["slug"]))
        info["is_panelist"] = bool(
            _panelist_utility.slug_exists(panelist_slug=info["slug"])
        )
        info["appearances"] = self.appearances.retrieve_appearances_by_id(
            scorekeeper_id
        )

        return info

    def retrieve_details_by_slug(self, scorekeeper_slug: str) -> dict[str, Any]:
        """Retrieves scorekeeper information and appearances.

        :param scorekeeper_slug: Scorekeeper slug string
        :return: A dictionaries containing scorekeeper ID, name, slug
            string, gender, pronouns, whether the scorekeeper is also
            a guest, host or panelist, and a list of appearances with
            show flags
        """
        try:
            slug = scorekeeper_slug.strip()
            if not slug:
                return {}
        except AttributeError:
            return {}

        id_ = self.utility.convert_slug_to_id(slug)
        if not id_:
            return {}

        return self.retrieve_details_by_id(id_)

    def retrieve_random_id(self) -> int:
        """Retrieves an ID for a random scorekeeper.

        :return: ID for a random scorekeeper.
        """
        query = """
            SELECT scorekeeperid FROM ww_scorekeepers
            WHERE scorekeeperslug <> 'tbd'
            ORDER BY RAND()
            LIMIT 1;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query)
        result = cursor.fetchone()
        cursor.close()

        if not result:
            return None

        return result[0]

    def retrieve_random_slug(self) -> str:
        """Retrieves an slug string for a random scorekeeper.

        :return: Slug string for a random scorekeeper.
        """
        query = """
            SELECT scorekeeperslug FROM ww_scorekeepers
            WHERE scorekeeperslug <> 'tbd'
            ORDER BY RAND()
            LIMIT 1;
            """
        cursor = self.database_connection.cursor(dictionary=False)
        cursor.execute(query)
        result = cursor.fetchone()
        cursor.close()

        if not result:
            return None

        return result[0]

    def retrieve_random(self) -> dict[str, Any]:
        """Retrieves information for a random scorekeeper.

        :return: A dictionary containing scorekeeper ID, name, slug
            string, gender and pronouns
        """
        _id = self.retrieve_random_id()

        if not _id:
            return None

        return self.retrieve_by_id(scorekeeper_id=_id)

    def retrieve_random_details(self) -> dict[str, Any]:
        """Retrieves information and appearances for a random scorekeeper.

        :return: A dictionaries containing scorekeeper ID, name, slug
            string, gender, pronouns, whether the scorekeeper is also
            a guest, host or panelist, and a list of appearances with
            show flags
        """
        _id = self.retrieve_random_id()

        if not _id:
            return None

        return self.retrieve_details_by_id(scorekeeper_id=_id)



# Copyright (c) 2018-2026 Linh Pham
# wwdtm is released under the terms of the Apache License 2.0
# SPDX-License-Identifier: Apache-2.0
#
# vim: set noai syntax=python ts=4 sw=4:
"""Testing for object :py:class:`wwdtm.show.ShowInfo`."""

import json
from pathlib import Path
from typing import Any

import pytest

from wwdtm.show import ShowInfoMultiple


@pytest.mark.skip
def get_connect_dict() -> dict[str, Any]:
    """Retrieves database connection settings.

    :return: A dictionary containing database connection
        settings as required by MySQL Connector/Python
    """
    file_path = Path.cwd() / "config.json"
    with file_path.open(mode="r", encoding="utf-8") as config_file:
        config_dict = json.load(config_file)
        if "database" in config_dict:
            return config_dict["database"]


@pytest.mark.parametrize("show_id", [319, 1083, 1162])
def test_show_info_retrieve_bluff_info_all(show_id: int):
    """Testing for :py:meth:`wwdtm.show.ShowInfoMultiple.retrieve_bluff_info_all`."""
    info = ShowInfoMultiple(connect_dict=get_connect_dict())
    bluffs = info.retrieve_bluff_info_all()

    assert isinstance(bluffs, dict), (
        "Bluff the Listener information for all shows could not be retrieved"
    )

    assert show_id in bluffs and isinstance(bluffs[show_id], list), (
        f"Bluff the Listener information was not returned for show ID {show_id}"
    )

    assert "chosen_panelist" in bluffs[show_id][0], (
        f"'chosen_panelist' was not returned with panelist information for show ID {show_id}"
    )

    assert "correct_panelist" in bluffs[show_id][0], (
        f"'correct_panelist' was not returned with panelist information for show ID {show_id}"
    )


@pytest.mark.parametrize("show_ids", [[319, 1083, 1162]])
def test_show_info_retrieve_bluff_info_by_ids(show_ids: list[int]):
    """Testing for :py:meth:`wwdtm.show.ShowInfoMultiple.retrieve_bluff_info_by_ids`.

    :param show_ids: List of show IDs to test retrieving show Bluff the
        Listener information
    """
    info = ShowInfoMultiple(connect_dict=get_connect_dict())
    bluffs = info.retrieve_bluff_info_by_ids(show_ids)

    assert bluffs, (
        f"Bluff the Listener information for the show IDs {show_ids} "
        "could not be retrieved"
    )

    for show_id in show_ids:
        assert show_id in bluffs and isinstance(bluffs[show_id], list), (
            f"Bluff the Listener information was not returned for show ID {show_id}"
        )

        assert "chosen_panelist" in bluffs[show_id][0], (
            "'chosen_panelist' was not returned with panelist information for show ID "
            f"{show_id}"
        )

        assert "correct_panelist" in bluffs[show_id][0], (
            "'correct_panelist' was not returned with panelist information for show "
            f"ID {show_id}"
        )


@pytest.mark.parametrize("show_id", [1162])
def test_show_info_retrieve_core_info_all(show_id: int):
    """Testing for :py:meth:`wwdtm.show.ShowInfoMultiple.retrieve_core_info_all`.

    :param show_id: Show ID to test retrieving show core information
        from all shows retrieved
    """
    info = ShowInfoMultiple(connect_dict=get_connect_dict())
    shows = info.retrieve_core_info_all()

    assert shows, "Core information for all shows could not be retrieved"
    assert show_id in shows, (
        f"Core information for show ID {show_id} could not be retrieved"
    )

    show = shows[show_id]
    assert "id" in show, (
        f"'id' was not returned with core information for show ID {show_id}"
    )
    assert "description" in show, (
        f"'description' was not returned with show information for show ID {show_id}"
    )


@pytest.mark.parametrize("show_ids", [[1082, 1162]])
def test_show_info_retrieve_core_info_by_ids(show_ids: list[int]):
    """Testing for :py:meth:`wwdtm.show.ShowInfoMultiple.retrieve_core_info_by_ids`.

    :param show_id: Show ID to test retrieving show core information
    """
    info = ShowInfoMultiple(connect_dict=get_connect_dict())
    shows = info.retrieve_core_info_by_ids(show_ids)

    assert shows, "Core information all shows could not be retrieved"

    for show_id in show_ids:
        assert show_id in shows, (
            f"Core information could not be retrieved for show ID {show_id}"
        )
        assert "id" in shows[show_id], (
            f"'id' was not returned with core information for show ID {show_id}"
        )
        assert "description" in shows[show_id], (
            "'description' was not returned with show information for show ID "
            f"{show_id}"
        )


@pytest.mark.parametrize("show_id", [1082])
def test_show_info_retrieve_guest_info_all(show_id: int):
    """Testing for :py:meth:`wwdtm.show.ShowInfoMultiple.retrieve_guest_info_all`.

    :param show_id: Show ID to test retrieving show guest information
        for all shows retrieved
    """
    info = ShowInfoMultiple(connect_dict=get_connect_dict())
    shows_guests = info.retrieve_guest_info_all()

    assert shows_guests, "Guest information all shows could not be retrieved"
    assert show_id in shows_guests, (
        f"Guest information could not be retrieved for show ID {show_id}"
    )

    guests = shows_guests[show_id]
    if guests:
        assert "id" in guests[0], (
            f"'id' was not returned for the first guest for show ID {show_id}"
        )
        assert "score" in guests[0], (
            f"'score' was not returned for the first guest for show ID {show_id}"
        )


@pytest.mark.parametrize("show_ids", [[1082, 1162]])
def test_show_info_retrieve_guest_info_by_ids(show_ids: list[int]):
    """Testing for :py:meth:`wwdtm.show.ShowInfoMultiple.retrieve_guest_info_by_ids`.

    :param show_ids: List of show IDs to test retrieving show guest
        information
    """
    info = ShowInfoMultiple(connect_dict=get_connect_dict())
    shows_guests = info.retrieve_guest_info_by_ids(show_ids)

    assert shows_guests, (
        f"Guest information for show IDs {show_ids} could not be retrieved"
    )

    for show_id in show_ids:
        assert show_id in shows_guests, (
            f"Guest information for show ID {show_id} could not be retrieved"
        )

        guests = shows_guests[show_id]
        if guests:
            assert "id" in guests[0], (
                f"'id' was not returned for the first guest for show ID {show_id}"
            )
            assert "score" in guests[0], (
                f"'score' was not returned for the first guest for show ID {show_id}"
            )


@pytest.mark.parametrize(
    "show_id, include_decimal_scores", [(1082, True), (1082, False)]
)
def test_show_info_retrieve_panelist_info_all(
    show_id: int, include_decimal_scores: bool
):
    """Testing for :py:meth:`wwdtm.show.ShowInfoMultiple.retrieve_panelist_info_all`.

    :param show_id: Show ID to test retrieving show panelist information
        for all shows retrieved
    :param include_decimal_scores: Flag set to include decimal score columns
        and values
    """
    info = ShowInfoMultiple(connect_dict=get_connect_dict())
    shows_panelists = info.retrieve_panelist_info_all(
        include_decimal_scores=include_decimal_scores
    )

    assert shows_panelists, "Panelist information for all shows could not be retrieved"
    assert show_id in shows_panelists, (
        f"Panelist information could not be retrieved for show ID {show_id}"
    )

    panelists = shows_panelists[show_id]
    assert "id" in panelists[0], (
        f"'id' was not returned for the first panelist for show ID {show_id}"
    )
    assert "score" in panelists[0], (
        f"'score' was not returned for the first panelist for show ID {show_id}"
    )
    if include_decimal_scores:
        assert "score_decimal" in panelists[0], (
            f"'score_decimal' was not returned for the first panelist for show ID {show_id}"
        )
    assert "score_exception" in panelists[0], (
        f"'score_exception' was not returned for the first panelist for show ID {show_id}"
    )


@pytest.mark.parametrize(
    "show_ids, include_decimal_scores", [([1082, 1162], True), ([1082, 1162], False)]
)
def test_show_info_retrieve_panelist_info_by_ids(
    show_ids: list[int], include_decimal_scores: bool
):
    """Testing for :py:meth:`wwdtm.show.ShowInfoMultiple.retrieve_panelist_info_by_ids`.

    :param show_ids: List of show IDs to test retrieving show panelist
        information
    :param include_decimal_scores: Flag set to include decimal score columns
        and values
    """
    info = ShowInfoMultiple(connect_dict=get_connect_dict())
    shows_panelists = info.retrieve_panelist_info_by_ids(
        show_ids, include_decimal_scores=include_decimal_scores
    )

    assert shows_panelists, (
        f"Panelist information for show IDs {show_ids} could not be retrieved"
    )

    for show_id in shows_panelists:
        assert show_id in shows_panelists, (
            f"Panelist information could not be retrieved for show ID {show_id}"
        )
        panelists = shows_panelists[show_id]
        if panelists:
            assert "id" in panelists[0], (
                f"'id' was not returned for the first panelist for show ID {show_id}"
            )
            assert "score" in panelists[0], (
                f"'score' was not returned for the first panelist for show ID {show_id}"
            )
            if include_decimal_scores:
                assert "score_decimal" in panelists[0], (
                    f"'score_decimal' was not returned for the first panelist for show ID {show_id}"
                )
            assert "score_exception" in panelists[0], (
                f"'score_exception' was not returned for the first panelist for show ID {show_id}"
            )
